@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -520,8 +520,11 @@ stringT PWScore::Merge(PWScore *pothercore,
           oth_pwp = PWSprefs::GetInstance()->GetDefaultPolicy(true);
         else
           otherItem.GetPWPolicy(oth_pwp);
-        if (cur_pwp != oth_pwp)
+        if (cur_pwp != oth_pwp) {
           diff_flags |= MRG_POLICY;
+          LoadAString(str_temp, IDSC_FLDNMPWPOLICY);
+          str_diffs += str_temp + _T(", ");
+        }
       }
 
       otherItem.GetXTime(oxt);
