@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -87,6 +87,7 @@ public:
     SYMBOLS = 0x16,    // string of item-specific password symbols
     SHIFTDCA = 0x17,   // shift-doubleclick action (enum)
     POLICYNAME = 0x18, // named non-default password policy for item
+    KBSHORTCUT = 0x19, // Keyboard shortcuts
     LAST,        // Start of unknown fields!
     END = 0xff,
     // Internal fields only - used in filters
@@ -119,14 +120,6 @@ public:
                     ES_DELETED      =  4,  // Deleted  but not yet removed from disk copy
                     ES_LAST};
 
-  // Flags if error found during validate of the entry
-  enum  {VF_OK              =  0,
-         VF_BAD_UUID        =  1,
-         VF_EMPTY_TITLE     =  2,
-         VF_EMPTY_PASSWORD  =  4,
-         VF_NOT_UNIQUE_GTU  =  8,
-         VF_BAD_PSWDHISTORY = 16};
-
   // a bitset for indicating a subset of an item's fields: 
   typedef std::bitset<LAST> FieldBits;
 
@@ -207,6 +200,8 @@ public:
   bool IsProtected() const;
   StringX GetSymbols() const    {return GetField(SYMBOLS);}
   StringX GetPolicyName() const {return GetField(POLICYNAME);}
+  void GetKBShortcut(int32 &iKBShortcut) const;
+  StringX GetKBShortcut() const;
 
   StringX GetFieldValue(FieldType ft) const;
 
@@ -215,7 +210,8 @@ public:
   StringX GetPlaintext(const TCHAR &separator, const FieldBits &bsExport,
                        const TCHAR &delimiter, const CItemData *pcibase) const;
   std::string GetXML(unsigned id, const FieldBits &bsExport, TCHAR m_delimiter,
-                     const CItemData *pcibase, bool bforce_normal_entry) const;
+                     const CItemData *pcibase, bool bforce_normal_entry,
+                     bool &bXMLErrorsFound) const;
 
   void SetUnknownField(unsigned char type, size_t length,
                        const unsigned char *ufield);
@@ -264,6 +260,8 @@ public:
   void SetProtected(bool bOnOff);
   void SetSymbols(const StringX &sx_symbols);
   void SetPolicyName(const StringX &sx_PolicyName);
+  void SetKBShortcut(const StringX &sx_KBShortcut);
+  void SetKBShortcut(const int32 &iKBShortcut);
 
   void SetFieldValue(FieldType ft, const StringX &value);
 
@@ -274,9 +272,7 @@ public:
   void Clear();
   void ClearField(FieldType ft) {m_fields.erase(ft);}
 
-
-  // check record for mandatory fields, silently fix if missing
-  bool ValidateEntry(int &flags);
+  // Check record for correct password history
   bool ValidatePWHistory(); // return true if OK, false if there's a problem
 
   bool IsExpired() const;
@@ -312,9 +308,11 @@ public:
   bool IsPasswordPolicySet() const         { return IsFieldSet(POLICY);    }
   bool IsPasswordExpiryIntervalSet() const { return IsFieldSet(XTIME_INT); }
   bool IsDCASet() const                    { return IsFieldSet(DCA);       }
+  bool IsShiftDCASet() const               { return IsFieldSet(SHIFTDCA);  }
   bool IsProtectionSet() const             { return IsFieldSet(PROTECTED); }
   bool IsSymbolsSet() const                { return IsFieldSet(SYMBOLS);   }
   bool IsPolicyNameSet() const             { return IsFieldSet(POLICYNAME);}
+  bool IsKBShortcutSet() const             { return IsFieldSet(KBSHORTCUT);}
     
   bool IsGroupEmpty() const                { return !IsGroupSet();         }
   bool IsUserEmpty() const                 { return !IsUserSet();          }
@@ -411,10 +409,12 @@ private:
 
 inline bool CItemData::IsTextField(unsigned char t)
 {
-  return !(t == UUID ||
-    t == CTIME     || t == PMTIME || t == ATIME || t == XTIME || t == RMTIME ||
-    t == XTIME_INT ||
-    t == RESERVED  || t == DCA    || t == SHIFTDCA || t == PROTECTED ||
+  return !(
+    t == UUID       ||
+    t == CTIME      || t == PMTIME || t == ATIME    || t == XTIME     || t == RMTIME ||
+    t == XTIME_INT  ||
+    t == RESERVED   || t == DCA    || t == SHIFTDCA || t == PROTECTED ||
+    t == KBSHORTCUT ||
     t >= LAST);
 }
 #endif /* __ITEMDATA_H */
