@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -14,8 +14,10 @@
 #ifdef _WIN32
 #include <io.h>
 #endif
+
 #include <fcntl.h>
 #include <sys/stat.h>
+#include <algorithm>
 
 PWSfileV1V2::PWSfileV1V2(const StringX &filename, RWmode mode, VERSION version)
   : PWSfile(filename, mode)
@@ -215,7 +217,8 @@ int PWSfileV1V2::CheckPasskey(const StringX &filename,
   unsigned char temphash[20]; // HashSize
   GenRandhash(passkey, randstuff, temphash);
 
-  if (0 != memcmp(reinterpret_cast<char *>(randhash), reinterpret_cast<char *>(temphash), MIN(sizeof(randhash), sizeof(temphash)))) { // HashSize
+  if (0 != memcmp(reinterpret_cast<char *>(randhash), reinterpret_cast<char *>(temphash),
+                     std::min(sizeof(randhash), sizeof(temphash)))) { // HashSize
       return WRONG_PASSWORD;
   } else {
     return SUCCESS;
