@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -14,6 +14,8 @@
 #include "PwsPlatform.h"
 #include "core.h"
 #include "StringXStream.h"
+#include "PWPolicy.h"
+
 #include "Util.h"
 
 #include "os/debug.h"
@@ -30,7 +32,7 @@
 using namespace std;
 
 // used by CBC routines...
-static void xormem(unsigned char* mem1, const unsigned char* mem2, int length)
+static void xormem(unsigned char *mem1, const unsigned char *mem2, int length)
 {
   for (int x = 0; x < length; x++)
     mem1[x] ^= mem2[x];
@@ -38,15 +40,15 @@ static void xormem(unsigned char* mem1, const unsigned char* mem2, int length)
 
 //-----------------------------------------------------------------------------
 //Overwrite the memory
-// used to be a loop here, but this was deemed (1) overly paranoid 
-// (2) The wrong way to scrub DRAM memory 
-// see http://www.cs.auckland.ac.nz/~pgut001/pubs/secure_del.html 
-// and http://www.cypherpunks.to/~peter/usenix01.pdf 
+// used to be a loop here, but this was deemed (1) overly paranoid
+// (2) The wrong way to scrub DRAM memory
+// see http://www.cs.auckland.ac.nz/~pgut001/pubs/secure_del.html
+// and http://www.cypherpunks.to/~peter/usenix01.pdf
 
 #ifdef _WIN32
 #pragma optimize("",off)
 #endif
-void trashMemory(void* buffer, size_t length)
+void trashMemory(void *buffer, size_t length)
 {
   ASSERT(buffer != NULL);
   // {kjp} no point in looping around doing nothing is there?
@@ -81,7 +83,7 @@ void ConvertString(const StringX &text,
                    unsigned char *&txt,
                    size_t &txtlen)
 {
-  LPCTSTR txtstr = text.c_str(); 
+  LPCTSTR txtstr = text.c_str();
   txtlen = text.length();
 
 #ifndef UNICODE
@@ -107,8 +109,8 @@ void ConvertString(const StringX &text,
 
 //Generates a passkey-based hash from stuff - used to validate the passkey
 void GenRandhash(const StringX &a_passkey,
-                 const unsigned char* a_randstuff,
-                 unsigned char* a_randhash)
+                 const unsigned char *a_randstuff,
+                 unsigned char *a_randhash)
 {
   size_t pkeyLen = 0;
   unsigned char *pstr = NULL;
@@ -150,8 +152,8 @@ void GenRandhash(const StringX &a_passkey,
   keyHash.Final(a_randhash);
 }
 
-size_t _writecbc(FILE *fp, const unsigned char* buffer, size_t length, unsigned char type,
-                 Fish *Algorithm, unsigned char* cbcbuffer)
+size_t _writecbc(FILE *fp, const unsigned char *buffer, size_t length, unsigned char type,
+                 Fish *Algorithm, unsigned char *cbcbuffer)
 {
   const unsigned int BS = Algorithm->GetBlockSize();
   size_t numWritten = 0;
@@ -239,8 +241,8 @@ size_t _writecbc(FILE *fp, const unsigned char* buffer, size_t length, unsigned
 */
 size_t _readcbc(FILE *fp,
          unsigned char* &buffer, size_t &buffer_len, unsigned char &type,
-         Fish *Algorithm, unsigned char* cbcbuffer,
-         const unsigned char *TERMINAL_BLOCK, size_t file_len)
+         Fish *Algorithm, unsigned char *cbcbuffer,
+         const unsigned char *TERMINAL_BLOCK, ulong64 file_len)
 {
   const unsigned int BS = Algorithm->GetBlockSize();
   size_t numRead = 0;
@@ -253,7 +255,7 @@ size_t _readcbc(FILE *fp,
   unsigned char *lengthblock = NULL;
 
   ASSERT(BS <= sizeof(block1)); // if needed we can be more sophisticated here...
-  
+
   // Safety check.  (Lockheed Martin) Secure Coding  11-14-2007
   if ((BS > sizeof( block1 )) || (BS == 0))
    return 0;
@@ -339,12 +341,8 @@ size_t _readcbc(FILE *fp,
 
 void PWSUtil::strCopy(LPTSTR target, size_t tcount, const LPCTSTR source, size_t scount)
 {
-#if (_MSC_VER >= 1400)
-  (void) _tcsncpy_s(target, tcount, source, scount);
-#else
-  tcount = 0; // shut up warning;
-  (void)_tcsncpy(target, source, scount);
-#endif
+  UNREFERENCED_PARAMETER(tcount); //not used in now in non-MSVS wrapped version of _tcsncpy_s
+  _tcsncpy_s(target, tcount, source, scount);
 }
 
 size_t PWSUtil::strLength(const LPCTSTR str)
@@ -355,51 +353,52 @@ size_t PWSUtil::strLength(const LPCTSTR str)
 const TCHAR *PWSUtil::UNKNOWN_XML_TIME_STR = _T("1970-01-01 00:00:00");
 const TCHAR *PWSUtil::UNKNOWN_ASC_TIME_STR = _T("Unknown");
 
-StringX PWSUtil::ConvertToDateTimeString(const time_t &t,
-                                         const int result_format)
+StringX PWSUtil::ConvertToDateTimeString(const time_t &t, TMC result_format)
 {
   StringX ret;
   if (t != 0) {
     TCHAR datetime_str[80];
     struct tm *st;
-#if (_MSC_VER >= 1400)
     struct tm st_s;
     errno_t err;
     err = localtime_s(&st_s, &t);  // secure version
     if (err != 0) // invalid time
       return ConvertToDateTimeString(0, result_format);
     st = &st_s; // hide difference between versions
-#else
-    st = localtime(&t);
-    if (st == NULL) // null means invalid time
-      return ConvertToDateTimeString(0, result_format);
-#endif
-    if ((result_format & TMC_EXPORT_IMPORT) == TMC_EXPORT_IMPORT)
+    switch (result_format) {
+    case TMC_EXPORT_IMPORT:
       _tcsftime(datetime_str, sizeof(datetime_str) / sizeof(datetime_str[0]),
                 _T("%Y/%m/%d %H:%M:%S"), st);
-    else if ((result_format & TMC_XML) == TMC_XML)
+      break;
+    case TMC_XML:
       _tcsftime(datetime_str, sizeof(datetime_str) / sizeof(datetime_str[0]),
                 _T("%Y-%m-%dT%H:%M:%S"), st);
-    else if ((result_format & TMC_LOCALE) == TMC_LOCALE) {
+      break;
+    case TMC_LOCALE:
       setlocale(LC_TIME, "");
       _tcsftime(datetime_str, sizeof(datetime_str) / sizeof(datetime_str[0]),
                 _T("%c"), st);
-    } else {
-      int terr = _tasctime_s(datetime_str, 32, st);  // secure version
-      if (terr != 0)
+      break;
+    case TMC_LOCALE_DATE_ONLY:
+      setlocale(LC_TIME, "");
+      _tcsftime(datetime_str, sizeof(datetime_str) / sizeof(datetime_str[0]),
+                _T("%x"), st);
+      break;
+    default:
+      if (_tasctime_s(datetime_str, 32, st) != 0)
         return ConvertToDateTimeString(0, result_format);
     }
     ret = datetime_str;
-  } else {
+  } else { // t == 0
     switch (result_format) {
-      case TMC_ASC_UNKNOWN:
-        ret = UNKNOWN_ASC_TIME_STR;
-        break;
-      case TMC_XML:
-        ret = UNKNOWN_XML_TIME_STR;
-        break;
-      default:
-        ret = _T("");
+    case TMC_ASC_UNKNOWN:
+      ret = UNKNOWN_ASC_TIME_STR;
+      break;
+    case TMC_XML:
+      ret = UNKNOWN_XML_TIME_STR;
+      break;
+    default:
+      ret = _T("");
     }
   }
   // remove the trailing EOL char.
@@ -421,17 +420,23 @@ stringT PWSUtil::GetNewFileName(const stringT &oldfilename,
   return outpath;
 }
 
-void PWSUtil::GetTimeStamp(stringT &sTimeStamp)
+stringT PWSUtil::GetTimeStamp(const bool bShort)
+{
+  stringT sTimeStamp;
+  GetTimeStamp(sTimeStamp, bShort);
+  return sTimeStamp;
+}
+
+void PWSUtil::GetTimeStamp(stringT &sTimeStamp, const bool bShort)
 {
   // Now re-entrant
+  // Gets datetime stamp in format YYYY/MM/DD HH:MM:SS.mmm
+  // If bShort == true, don't add milli-seconds
+
 #ifdef _WIN32
   struct _timeb *ptimebuffer;
   ptimebuffer = new _timeb;
-#if (_MSC_VER >= 1400)
   _ftime_s(ptimebuffer);
-#else
-  _ftime(ptimebuffer);
-#endif
 #else
   struct timeb *ptimebuffer;
   ptimebuffer = new timeb;
@@ -439,25 +444,29 @@ void PWSUtil::GetTimeStamp(stringT &sTimeStamp)
 #endif
   StringX cmys_now = ConvertToDateTimeString(ptimebuffer->time, TMC_EXPORT_IMPORT);
 
-  ostringstreamT *p_os;
-  p_os = new ostringstreamT;
-  *p_os << cmys_now << TCHAR('.') << setw(3) << setfill(TCHAR('0'))
-     << static_cast<unsigned int>(ptimebuffer->millitm);
+  if (bShort) {
+    sTimeStamp = cmys_now.c_str();
+  } else {
+    ostringstreamT *p_os;
+    p_os = new ostringstreamT;
+    *p_os << cmys_now << TCHAR('.') << setw(3) << setfill(TCHAR('0'))
+          << static_cast<unsigned int>(ptimebuffer->millitm);
 
-  sTimeStamp = p_os->str();
+    sTimeStamp = p_os->str();
+    delete p_os;
+  }
   delete ptimebuffer;
-  delete p_os;
 }
 
 stringT PWSUtil::Base64Encode(const BYTE *strIn, size_t len)
 {
   stringT cs_Out;
-  static const TCHAR base64ABC[] = 
+  static const TCHAR base64ABC[] =
     _S("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/");
 
   for (size_t i = 0; i < len; i += 3) {
-    long l = ( static_cast<long>(strIn[i]) << 16 ) | 
-               (((i + 1) < len) ? (static_cast<long>(strIn[i + 1]) << 8) : 0) | 
+    long l = ( static_cast<long>(strIn[i]) << 16 ) |
+               (((i + 1) < len) ? (static_cast<long>(strIn[i + 1]) << 8) : 0) |
                (((i + 2) < len) ? static_cast<long>(strIn[i + 2]) : 0);
 
     cs_Out += base64ABC[(l >> 18) & 0x3F];
@@ -477,7 +486,7 @@ stringT PWSUtil::Base64Encode(const BYTE *strIn, size_t len)
   return cs_Out;
 }
 
-void PWSUtil::Base64Decode(const StringX &inString, BYTE* &outData, size_t &out_len)
+void PWSUtil::Base64Decode(const StringX &inString, BYTE * &outData, size_t &out_len)
 {
   static const char szCS[]=
     "=ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
@@ -520,27 +529,73 @@ void PWSUtil::Base64Decode(const StringX &inString, BYTE* &outData, size_t &out_
   out_len = st_length;
 }
 
-static const size_t MAX_TTT_LEN = 64; // Max tooltip text length
-StringX PWSUtil::NormalizeTTT(const StringX &in)
+StringX PWSUtil::NormalizeTTT(const StringX &in, size_t maxlen)
 {
   StringX ttt;
-  if (in.length() >= MAX_TTT_LEN) {
-    ttt = in.substr(0, MAX_TTT_LEN/2-6) + 
-      _T(" ... ") + in.substr(in.length() - MAX_TTT_LEN/2);
+  if (in.length() >= maxlen) {
+    ttt = in.substr(0, maxlen/2-6) +
+      _T(" ... ") + in.substr(in.length() - maxlen/2);
   } else {
     ttt = in;
   }
   return ttt;
 }
 
-void PWSUtil::WriteXMLField(ostream &os, const char *fname,
+bool ValidateXMLCharacters(const StringX &value, ostringstream &ostInvalidPositions)
+{
+  // From: http://www.w3.org/TR/REC-xml/#sec-cdata-sect
+  // CDATA Sections
+  // [18]    CDSect   ::=    CDStart CData CDEnd
+  // [19]    CDStart  ::=    '<![CDATA['
+  // [20]    CData    ::=    (Char* - (Char* ']]>' Char*))
+  // [21]    CDEnd    ::=    ']]>'
+
+  // From: http://www.w3.org/TR/REC-xml/#NT-Char
+  //  Char    ::=    #x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD] |
+  //                 [#x10000-#x10FFFF]
+  // any Unicode character, excluding the surrogate blocks, FFFE, and FFFF.
+
+  // Easy to check low values (0x00-0x1f excluding 3 above) and the 2 special values
+  // Not so easy for the range 0xD800 to 0xDFFF (Unicode only) - could use regex
+  // but expected to be slower than this!
+
+  ostInvalidPositions.str("");
+  bool bInvalidFound(false);
+  for (size_t i = 0; i < value.length(); i++) {
+    TCHAR current = value[i];
+    if (!((current == 0x09) ||
+          (current == 0x0A) ||
+          (current == 0x0D) ||
+          ((current >=    0x20) && (current <=   0xD7FF)) ||
+          ((current >=  0xE000) && (current <=   0xFFFD)) ||
+          ((current >= 0x10000) && (current <= 0x10FFFF)))) {
+      if (bInvalidFound) {
+        // Already 1 position, add a comma
+        ostInvalidPositions << ", ";
+      }
+      bInvalidFound = true;
+      ostInvalidPositions << (i + 1);
+    }
+  }
+  return !bInvalidFound;
+}
+
+bool PWSUtil::WriteXMLField(ostream &os, const char *fname,
                             const StringX &value, CUTF8Conv &utf8conv,
                             const char *tabs)
 {
-  const unsigned char * utf8 = NULL;
+  const unsigned char *utf8 = NULL;
   size_t utf8Len = 0;
-  string::size_type p = value.find(_T("]]>")); // special handling required
-  if (p == string::npos) {
+  ostringstream ostInvalidPositions;
+  if (!ValidateXMLCharacters(value, ostInvalidPositions)) {
+    os << tabs << "<!-- Field '<" << fname << ">' contains invalid XML character(s)" << endl;
+    os << tabs << "   at position(s): " << ostInvalidPositions.str().c_str() << endl;
+    os << tabs << "   and has been skipped -->" << endl;
+    return false;
+  }
+
+  StringX::size_type p = value.find(_T("]]>")); // special handling required
+  if (p == StringX::npos) {
     // common case
     os << tabs << "<" << fname << "><![CDATA[";
     if (utf8conv.ToUTF8(value, utf8, utf8Len))
@@ -564,7 +619,7 @@ void PWSUtil::WriteXMLField(ostream &os, const char *fname,
       os << "]]><![CDATA[";
       from = to;
       p = value.find(_T("]]>"), from); // are there more?
-      if (p == string::npos) {
+      if (p == StringX::npos) {
         to = value.length();
         slice = value.substr(from, (to - from));
       } else {
@@ -581,6 +636,7 @@ void PWSUtil::WriteXMLField(ostream &os, const char *fname,
     } while (p != StringX::npos);
     os << "</" << fname << ">" << endl;
   } // special handling of "]]>" in value.
+  return true;
 }
 
 string PWSUtil::GetXMLTime(int indent, const char *name,
@@ -594,18 +650,12 @@ string PWSUtil::GetXMLTime(int indent, const char *name,
 
 
   for (i = 0; i < indent; i++) oss << "\t";
-  oss << "<" << name << ">" << endl;
-  for (i = 0; i <= indent; i++) oss << "\t";
+  oss << "<" << name << ">" ;
   utf8conv.ToUTF8(tmp.substr(0, 10), utf8, utf8Len);
-  oss << "<date>";
   oss.write(reinterpret_cast<const char *>(utf8), utf8Len);
-  oss << "</date>" << endl;
-  for (i = 0; i <= indent; i++) oss << "\t";
+  oss << "T";
   utf8conv.ToUTF8(tmp.substr(tmp.length() - 8), utf8, utf8Len);
-  oss << "<time>";
   oss.write(reinterpret_cast<const char *>(utf8), utf8Len);
-  oss << "</time>" << endl;
-  for (i = 0; i < indent; i++) oss << "\t";
   oss << "</" << name << ">" << endl;
   return oss.str();
 }
@@ -628,7 +678,7 @@ int GetStringBufSize(const TCHAR *fmt, va_list args)
   va_list ar;
   va_copy(ar, args);
   // Linux doesn't do this correctly :-(
-  int guess = 16;
+  int guess = 64;
   while (1) {
     len = guess;
     buffer = new TCHAR[len];
@@ -639,6 +689,7 @@ int GetStringBufSize(const TCHAR *fmt, va_list args)
       break;
     else { // too small, resize & try again
       delete[] buffer;
+      buffer = NULL;
       guess *= 2;
     }
   }
@@ -646,7 +697,8 @@ int GetStringBufSize(const TCHAR *fmt, va_list args)
 #endif
   if (buffer)
     delete[] buffer;
-  ASSERT(len>0);
+
+  ASSERT(len > 0);
   return len;
 }
 
@@ -670,4 +722,49 @@ StringX PWSUtil::DeDupString(StringX &in_string)
     c++;
   }
   return out_string;
-}
\ No newline at end of file
+}
+
+stringT PWSUtil::GetSafeXMLString(const StringX &sxInString)
+{
+  stringT retval(_T(""));
+  ostringstreamT os;
+
+  StringX::size_type p = sxInString.find(_T("]]>")); // special handling required
+  if (p == StringX::npos) {
+    // common case
+    os << "<![CDATA[" << sxInString << "]]>";
+  } else {
+    // value has "]]>" sequence(s) that need(s) to be escaped
+    // Each "]]>" splits the field into two CDATA sections, one ending with
+    // ']]', the other starting with '>'
+    const StringX value = sxInString;
+    size_t from = 0, to = p + 2;
+    do {
+      StringX slice = value.substr(from, (to - from));
+      os << "<![CDATA[" << slice << "]]><![CDATA[";
+      from = to;
+      p = value.find(_T("]]>"), from); // are there more?
+      if (p == StringX::npos) {
+        to = value.length();
+        slice = value.substr(from, (to - from));
+      } else {
+        to = p + 2;
+        slice = value.substr(from, (to - from));
+        from = to;
+        to = value.length();
+      }
+      os <<  slice << "]]>";
+    } while (p != StringX::npos);
+  }
+  retval = os.str().c_str();
+  return retval;
+}
+
+bool operator==(const std::string& str1, const stringT& str2)
+{
+    CUTF8Conv conv;
+    StringX xstr;
+    VERIFY( conv.FromUTF8( reinterpret_cast<const unsigned char*>(str1.data()), str1.size(), xstr) );
+    return stringx2std(xstr) == str2;
+}
+
