@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -27,15 +27,15 @@
 
 #include <algorithm>
 
-extern const TCHAR *FORMATIMPORTED;
+extern const TCHAR *GROUPTITLEUSERINCHEVRONS;
 
 using namespace std;
 using pws_os::CUUID;
 
 XMLFileHandlers::XMLFileHandlers()
 {
-  cur_entry = NULL;
-  cur_pwhistory_entry = NULL;
+  m_cur_entry = NULL;
+  m_cur_pwhistory_entry = NULL;
   m_sxElemContent = _T("");
 
   m_delimiter = _T('\0');
@@ -48,14 +48,12 @@ XMLFileHandlers::XMLFileHandlers()
   m_bRecordHeaderErrors = false;
   m_bErrors = false;
 
-  m_nITER = MIN_HASH_ITERATIONS;
-  
   m_sxXML_DateTime = PWSUtil::GetTimeStamp(true).c_str();
 
   // Set up copy of preferences to use for password policies and if we import into an
   // empty database - Note: Load of empty string sets it to all defaults
   PWSprefs::GetInstance()->Load(_T(""), true);
-  
+
   // Get current DB default policy (any import DB default policy will be set after all
   // the preferences are read in)
   currentDB_default_pwp = PWSprefs::GetInstance()->GetDefaultPolicy();
@@ -93,6 +91,7 @@ bool XMLFileHandlers::ProcessStartElement(const int icurrent_element)
       m_numEntriesPWHErrors = 0;
       m_numNoPolicies = 0;
       m_numRenamedPolicies = 0;
+      m_numShortcutsRemoved = 0;
       m_bEntryBeingProcessed = false;
       break;
     case XLE_ENTRY:
@@ -100,43 +99,44 @@ bool XMLFileHandlers::ProcessStartElement(const int icurrent_element)
       if (m_bValidation)
         return false;
 
-      cur_entry = new pw_entry;
-      // Clear all fields
-      cur_entry->id = 0;
-      cur_entry->group = _T("");
-      cur_entry->title = _T("");
-      cur_entry->username = _T("");
-      cur_entry->password = _T("");
-      cur_entry->url = _T("");
-      cur_entry->autotype = _T("");
-      cur_entry->ctime = _T("");
-      cur_entry->atime = _T("");
-      cur_entry->xtime = _T("");
-      cur_entry->xtime_interval = _T("");
-      cur_entry->pmtime = _T("");
-      cur_entry->rmtime = _T("");
-      cur_entry->pwhistory = _T("");
-      cur_entry->notes = _T("");
-      cur_entry->uuid = _T("");
-      cur_entry->pwp.Empty();
-      cur_entry->run_command = _T("");
-      cur_entry->dca = _T("");
-      cur_entry->shiftdca = _T("");
-      cur_entry->email = _T("");
-      cur_entry->symbols = _T("");
-      cur_entry->policyname = _T("");
-      cur_entry->ucprotected = 0;
-      cur_entry->entrytype = NORMAL;
-      cur_entry->bforce_normal_entry = false;
+      m_cur_entry = new pw_entry;
+      // Clear all fields XXX Do we need this? The default c'tor should handle this for us
+      m_cur_entry->id = 0;
+      m_cur_entry->group = _T("");
+      m_cur_entry->title = _T("");
+      m_cur_entry->username = _T("");
+      m_cur_entry->password = _T("");
+      m_cur_entry->url = _T("");
+      m_cur_entry->autotype = _T("");
+      m_cur_entry->ctime = _T("");
+      m_cur_entry->atime = _T("");
+      m_cur_entry->xtime = _T("");
+      m_cur_entry->xtime_interval = _T("");
+      m_cur_entry->pmtime = _T("");
+      m_cur_entry->rmtime = _T("");
+      m_cur_entry->pwhistory = _T("");
+      m_cur_entry->notes = _T("");
+      m_cur_entry->uuid = _T("");
+      m_cur_entry->pwp.Empty();
+      m_cur_entry->run_command = _T("");
+      m_cur_entry->dca = _T("");
+      m_cur_entry->shiftdca = _T("");
+      m_cur_entry->email = _T("");
+      m_cur_entry->symbols = _T("");
+      m_cur_entry->policyname = _T("");
+      m_cur_entry->kbshortcut = _T("");
+      m_cur_entry->ucprotected = 0;
+      m_cur_entry->entrytype = NORMAL;
+      m_cur_entry->bforce_normal_entry = false;
       break;
     case XLE_HISTORY_ENTRY:
       if (m_bValidation)
         return false;
 
-      ASSERT(cur_pwhistory_entry == NULL);
-      cur_pwhistory_entry = new pwhistory_entry;
-      cur_pwhistory_entry->changed = _T("");
-      cur_pwhistory_entry->oldpassword = _T("");
+      ASSERT(m_cur_pwhistory_entry == NULL);
+      m_cur_pwhistory_entry = new pwhistory_entry;
+      m_cur_pwhistory_entry->changed = _T("");
+      m_cur_pwhistory_entry->oldpassword = _T("");
       break;
     case XLE_CTIMEX:
     case XLE_ATIMEX:
@@ -178,14 +178,8 @@ void XMLFileHandlers::ProcessEndElement(const int icurrent_element)
   PWSprefs::StringPrefs spref(PWSprefs::NumStringPrefs);
 
   switch (icurrent_element) {
-    case XLE_NUMBERHASHITERATIONS:
-      i = _ttoi(m_sxElemContent.c_str());
-      if (i > MIN_HASH_ITERATIONS) {
-        m_nITER = i;
-      }
-      break;
     case XLE_ENTRY:
-      m_ventries.push_back(cur_entry);
+      m_ventries.push_back(m_cur_entry);
       m_numEntries++;
       break;
 
@@ -364,170 +358,173 @@ void XMLFileHandlers::ProcessEndElement(const int icurrent_element)
 
     // MUST be in the same order as enum beginning STR_GROUP...
     case XLE_GROUP:
-      cur_entry->group = m_sxElemContent;
+      m_cur_entry->group = m_sxElemContent;
       break;
     case XLE_TITLE:
-      cur_entry->title = m_sxElemContent;
+      m_cur_entry->title = m_sxElemContent;
       break;
     case XLE_USERNAME:
-      cur_entry->username = m_sxElemContent;
+      m_cur_entry->username = m_sxElemContent;
       break;
     case XLE_URL:
-      cur_entry->url = m_sxElemContent;
+      m_cur_entry->url = m_sxElemContent;
       break;
     case XLE_AUTOTYPE:
-      cur_entry->autotype = m_sxElemContent;
+      m_cur_entry->autotype = m_sxElemContent;
       break;
     case XLE_NOTES:
-      cur_entry->notes = m_sxElemContent;
+      m_cur_entry->notes = m_sxElemContent;
       break;
     case XLE_UUID:
-      cur_entry->uuid = m_sxElemContent;
+      m_cur_entry->uuid = m_sxElemContent;
       break;
     case XLE_PASSWORD:
-      cur_entry->password = m_sxElemContent;
+      m_cur_entry->password = m_sxElemContent;
       if (Replace(m_sxElemContent, _T(':'), _T(';')) <= 2) {
         if (m_sxElemContent.substr(0, 2) == _T("[[") &&
             m_sxElemContent.substr(m_sxElemContent.length() - 2) == _T("]]")) {
-            cur_entry->entrytype = ALIAS;
+            m_cur_entry->entrytype = ALIAS;
         }
         if (m_sxElemContent.substr(0, 2) == _T("[~") &&
             m_sxElemContent.substr(m_sxElemContent.length() - 2) == _T("~]")) {
-            cur_entry->entrytype = SHORTCUT;
+            m_cur_entry->entrytype = SHORTCUT;
         }
       }
       break;
     case XLE_CTIMEX:
-      cur_entry->ctime = m_sxElemContent;
+      m_cur_entry->ctime = m_sxElemContent;
       break;
     case XLE_ATIMEX:
-      cur_entry->atime = m_sxElemContent;
+      m_cur_entry->atime = m_sxElemContent;
       break;
     case XLE_XTIMEX:
-      cur_entry->xtime = m_sxElemContent;
+      m_cur_entry->xtime = m_sxElemContent;
       break;
     case XLE_PMTIMEX:
-      cur_entry->pmtime = m_sxElemContent;
+      m_cur_entry->pmtime = m_sxElemContent;
       break;
     case XLE_RMTIMEX:
-      cur_entry->rmtime = m_sxElemContent;
+      m_cur_entry->rmtime = m_sxElemContent;
       break;
     case XLE_XTIME_INTERVAL:
-      cur_entry->xtime_interval = Trim(m_sxElemContent);
+      m_cur_entry->xtime_interval = Trim(m_sxElemContent);
       break;
     case XLE_RUNCOMMAND:
-      cur_entry->run_command = m_sxElemContent;
+      m_cur_entry->run_command = m_sxElemContent;
       break;
     case XLE_DCA:
-      cur_entry->dca = Trim(m_sxElemContent);
+      m_cur_entry->dca = Trim(m_sxElemContent);
       break;
     case XLE_SHIFTDCA:
-      cur_entry->shiftdca = Trim(m_sxElemContent);
+      m_cur_entry->shiftdca = Trim(m_sxElemContent);
       break;
     case XLE_EMAIL:
-      cur_entry->email = m_sxElemContent;
+      m_cur_entry->email = m_sxElemContent;
       break;
     case XLE_PROTECTED:
       if (m_sxElemContent == _T("1"))
-        cur_entry->ucprotected = 1;
+        m_cur_entry->ucprotected = 1;
       break;
     case XLE_SYMBOLS:
       if (m_bPolicyBeingProcessed)
         m_Named_pwp.symbols = m_sxElemContent;
       else
-        cur_entry->symbols = m_sxElemContent;
+        m_cur_entry->symbols = m_sxElemContent;
       break;
     case XLE_ENTRY_PASSWORDPOLICYNAME:
-      cur_entry->policyname = m_sxElemContent;
+      m_cur_entry->policyname = m_sxElemContent;
+      break;
+    case XLE_KBSHORTCUT:
+      m_cur_entry->kbshortcut = m_sxElemContent;
       break;
     case XLE_STATUS:
       i = _ttoi(m_sxElemContent.c_str());
       Format(buffer, _T("%01x"), i);
-      cur_entry->pwhistory = buffer;
+      m_cur_entry->pwhistory = buffer;
       break;
     case XLE_MAX:
       i = _ttoi(m_sxElemContent.c_str());
       Format(buffer, _T("%02x"), i);
-      cur_entry->pwhistory += buffer;
+      m_cur_entry->pwhistory += buffer;
       break;
     case XLE_NUM:
       i = _ttoi(m_sxElemContent.c_str());
       Format(buffer, _T("%02x"), i);
-      cur_entry->pwhistory += buffer;
+      m_cur_entry->pwhistory += buffer;
       break;
     case XLE_HISTORY_ENTRY:
-      ASSERT(cur_pwhistory_entry != NULL);
+      ASSERT(m_cur_pwhistory_entry != NULL);
       Format(buffer, _T("\xff%s\xff%04x\xff%s"),
-             cur_pwhistory_entry->changed.c_str(),
-             cur_pwhistory_entry->oldpassword.length(),
-             cur_pwhistory_entry->oldpassword.c_str());
-      cur_entry->pwhistory += buffer;
-      delete cur_pwhistory_entry;
-      cur_pwhistory_entry = NULL;
+             m_cur_pwhistory_entry->changed.c_str(),
+             m_cur_pwhistory_entry->oldpassword.length(),
+             m_cur_pwhistory_entry->oldpassword.c_str());
+      m_cur_entry->pwhistory += buffer;
+      delete m_cur_pwhistory_entry;
+      m_cur_pwhistory_entry = NULL;
       break;
     case XLE_CHANGEDX:
-      cur_pwhistory_entry->changed = m_sxElemContent;
+      m_cur_pwhistory_entry->changed = m_sxElemContent;
       break;
     case XLE_OLDPASSWORD:
-      ASSERT(cur_pwhistory_entry != NULL);
-      cur_pwhistory_entry->oldpassword = m_sxElemContent;
+      ASSERT(m_cur_pwhistory_entry != NULL);
+      m_cur_pwhistory_entry->oldpassword = m_sxElemContent;
       break;
     case XLE_ENTRY_PWLENGTH:
-      cur_entry->pwp.length = _ttoi(m_sxElemContent.c_str());
+      m_cur_entry->pwp.length = _ttoi(m_sxElemContent.c_str());
       break;
     case XLE_ENTRY_PWUSEDIGITS:
       if (m_sxElemContent == _T("1"))
-        cur_entry->pwp.flags |= PWPolicy::UseDigits;
+        m_cur_entry->pwp.flags |= PWPolicy::UseDigits;
       else
-        cur_entry->pwp.flags &= ~PWPolicy::UseDigits;
+        m_cur_entry->pwp.flags &= ~PWPolicy::UseDigits;
       break;
     case XLE_ENTRY_PWUSEEASYVISION:
       if (m_sxElemContent == _T("1"))
-        cur_entry->pwp.flags |= PWPolicy::UseEasyVision;
+        m_cur_entry->pwp.flags |= PWPolicy::UseEasyVision;
       else
-        cur_entry->pwp.flags &= ~PWPolicy::UseEasyVision;
+        m_cur_entry->pwp.flags &= ~PWPolicy::UseEasyVision;
       break;
     case XLE_ENTRY_PWUSEHEXDIGITS:
       if (m_sxElemContent == _T("1"))
-        cur_entry->pwp.flags |= PWPolicy::UseHexDigits;
+        m_cur_entry->pwp.flags |= PWPolicy::UseHexDigits;
       else
-        cur_entry->pwp.flags &= ~PWPolicy::UseHexDigits;
+        m_cur_entry->pwp.flags &= ~PWPolicy::UseHexDigits;
       break;
     case XLE_ENTRY_PWUSELOWERCASE:
       if (m_sxElemContent == _T("1"))
-        cur_entry->pwp.flags |= PWPolicy::UseLowercase;
+        m_cur_entry->pwp.flags |= PWPolicy::UseLowercase;
       else
-        cur_entry->pwp.flags &= ~PWPolicy::UseLowercase;
+        m_cur_entry->pwp.flags &= ~PWPolicy::UseLowercase;
       break;
     case XLE_ENTRY_PWUSESYMBOLS:
       if (m_sxElemContent == _T("1"))
-        cur_entry->pwp.flags |= PWPolicy::UseSymbols;
+        m_cur_entry->pwp.flags |= PWPolicy::UseSymbols;
       else
-        cur_entry->pwp.flags &= ~PWPolicy::UseSymbols;
+        m_cur_entry->pwp.flags &= ~PWPolicy::UseSymbols;
       break;
     case XLE_ENTRY_PWUSEUPPERCASE:
       if (m_sxElemContent == _T("1"))
-        cur_entry->pwp.flags |= PWPolicy::UseUppercase;
+        m_cur_entry->pwp.flags |= PWPolicy::UseUppercase;
       else
-        cur_entry->pwp.flags &= ~PWPolicy::UseUppercase;
+        m_cur_entry->pwp.flags &= ~PWPolicy::UseUppercase;
       break;
     case XLE_ENTRY_PWMAKEPRONOUNCEABLE:
       if (m_sxElemContent == _T("1"))
-        cur_entry->pwp.flags |= PWPolicy::MakePronounceable;
+        m_cur_entry->pwp.flags |= PWPolicy::MakePronounceable;
       else
-        cur_entry->pwp.flags &= ~PWPolicy::MakePronounceable;
+        m_cur_entry->pwp.flags &= ~PWPolicy::MakePronounceable;
       break;
     case XLE_ENTRY_PWDIGITMINLENGTH:
-      cur_entry->pwp.digitminlength = _ttoi(m_sxElemContent.c_str());
+      m_cur_entry->pwp.digitminlength = _ttoi(m_sxElemContent.c_str());
       break;
     case XLE_ENTRY_PWLOWERCASEMINLENGTH:
-      cur_entry->pwp.lowerminlength = _ttoi(m_sxElemContent.c_str());
+      m_cur_entry->pwp.lowerminlength = _ttoi(m_sxElemContent.c_str());
       break;
     case XLE_ENTRY_PWSYMBOLMINLENGTH:
-      cur_entry->pwp.symbolminlength = _ttoi(m_sxElemContent.c_str());
+      m_cur_entry->pwp.symbolminlength = _ttoi(m_sxElemContent.c_str());
       break;
     case XLE_ENTRY_PWUPPERCASEMINLENGTH:
-      cur_entry->pwp.upperminlength = _ttoi(m_sxElemContent.c_str());
+      m_cur_entry->pwp.upperminlength = _ttoi(m_sxElemContent.c_str());
       break;
     case XLE_PASSWORDSAFE:
     case XLE_PWHISTORY:
@@ -560,7 +557,7 @@ void XMLFileHandlers::ProcessEndElement(const int icurrent_element)
       m_PolicyName = m_sxElemContent.c_str();
       break;
   }
-  
+
   // If we have processed a DB preference - add it to our copy
   if (bpref != PWSprefs::NumBoolPrefs)    // boolean
     PWSprefs::GetInstance()->SetPref(bpref, _ttoi(m_sxElemContent.c_str()) == 0 ? false : true, true);
@@ -579,7 +576,7 @@ void XMLFileHandlers::AddXMLEntries()
                             DBPolicyNamesCommand::NP_ADDNEW);
     m_pmulticmds->Add(pcmd);
   }
-  
+
   // Then add any Empty Groups imported that are not already in the database
   if (!m_vEmptyGroups.empty()) {
     Command *pcmd = DBEmptyGroupsCommand::Create(m_pXMLcore, m_vEmptyGroups,
@@ -587,13 +584,13 @@ void XMLFileHandlers::AddXMLEntries()
     m_pmulticmds->Add(pcmd);
   }
 
-  // Gte current DB default password policy and that from the XML file and
+  // Get current DB default password policy and that from the XML file and
   // check that they are the same?
   PWPolicy st_to_default_pp, st_import_default_pp;
   st_to_default_pp = PWSprefs::GetInstance()->GetDefaultPolicy();
   st_import_default_pp = PWSprefs::GetInstance()->GetDefaultPolicy(true);
   const bool bPWPDefaults_Different = st_to_default_pp != st_import_default_pp;
-        
+
   StringX sxEntriesWithNewNamedPolicies;
   vdb_entries::iterator entry_iter;
   CItemData ci_temp;
@@ -636,7 +633,7 @@ void XMLFileHandlers::AddXMLEntries()
       cs_tp.erase(new_end, cs_tp.end());
 
       LoadAString(cs_id, IDSC_IMPORT_ENTRY_ID);
-      Format(cs_temp, IDSC_IMPORTENTRY, cs_id.c_str(), cur_entry->id, 
+      Format(cs_temp, IDSC_IMPORTENTRY, cs_id.c_str(), cur_entry->id,
              cur_entry->group.c_str(), cur_entry->title.c_str(), cur_entry->username.c_str());
       Format(cs_error, IDSC_IMPORTRECSKIPPED, cs_temp.c_str(), cs_tp.c_str());
       m_strSkippedList += cs_error;
@@ -645,13 +642,13 @@ void XMLFileHandlers::AddXMLEntries()
       delete cur_entry;
       continue;
     }
- 
+
     if (m_bImportPSWDsOnly) {
       ItemListIter iter = m_pXMLcore->Find(cur_entry->group, cur_entry->title, cur_entry->username);
       if (iter == m_pXMLcore->GetEntryEndIter()) {
         stringT cs_error, cs_id, cs_temp;
         LoadAString(cs_id, IDSC_IMPORT_ENTRY_ID);
-        Format(cs_temp, IDSC_IMPORTENTRY, cs_id.c_str(), cur_entry->id, 
+        Format(cs_temp, IDSC_IMPORTENTRY, cs_id.c_str(), cur_entry->id,
                cur_entry->group.c_str(), cur_entry->title.c_str(), cur_entry->username.c_str());
         Format(cs_error, IDSC_IMPORTRECNOTFOUND, cs_temp.c_str());
 
@@ -708,7 +705,8 @@ void XMLFileHandlers::AddXMLEntries()
     StringX sxnewgroup, sxnewtitle(cur_entry->title);
     if (!m_ImportedPrefix.empty()) {
       sxnewgroup = m_ImportedPrefix.c_str();
-      sxnewgroup += _T(".");
+      if (!cur_entry->group.empty())
+         sxnewgroup += _T(".");
     }
     sxnewgroup += cur_entry->group;
     EmptyIfOnlyWhiteSpace(sxnewgroup);
@@ -725,7 +723,7 @@ void XMLFileHandlers::AddXMLEntries()
         LoadAString(cs_header, IDSC_IMPORTCONFLICTSX2);
       else
         Format(cs_header, IDSC_IMPORTCONFLICTSX1, cur_entry->group.c_str());
-      
+
       Format(cs_error, IDSC_IMPORTCONFLICTS0, cs_header.c_str(),
                cur_entry->title.c_str(), cur_entry->username.c_str(), sxnewtitle.c_str());
       m_strRenameList += cs_error;
@@ -823,8 +821,8 @@ void XMLFileHandlers::AddXMLEntries()
           // Yes we did, so use renamed version
           cur_entry->policyname = citer->second;
           StringX sxChanged = L"\r\n\xab" + cur_entry->group    + L"\xbb " +
-	                            L"\xab"     + cur_entry->title    + L"\xbb " +
-	                            L"\xab"     + cur_entry->username + L"\xbb";
+                              L"\xab"     + cur_entry->title    + L"\xbb " +
+                              L"\xab"     + cur_entry->username + L"\xbb";
           sxEntriesWithNewNamedPolicies += sxChanged;
         } else {
           // No we didn't, verify current database has it
@@ -843,6 +841,10 @@ void XMLFileHandlers::AddXMLEntries()
       ci_temp.SetPolicyName(cur_entry->policyname);
     }
 
+    if (!cur_entry->kbshortcut.empty()) {
+      ci_temp.SetKBShortcut(cur_entry->kbshortcut);
+    }
+
     StringX newPWHistory;
     stringT strPWHErrorList;
 
@@ -858,7 +860,7 @@ void XMLFileHandlers::AddXMLEntries()
       case PWH_INVALID_NUM:
       case PWH_INVALID_DATETIME:
       case PWH_PSWD_LENGTH_NOTHEX:
-      case PWH_INVALID_PSWD_LENGTH: 
+      case PWH_INVALID_PSWD_LENGTH:
       case PWH_INVALID_FIELD_LENGTH:
       {
         stringT buffer;
@@ -890,16 +892,58 @@ void XMLFileHandlers::AddXMLEntries()
       ci_temp.SetStatus(CItemData::ES_ADDED);
     }
 
-    StringX sx_imported;
-    Format(sx_imported, FORMATIMPORTED,
-                        cur_entry->group.c_str(), cur_entry->title.c_str(), cur_entry->username.c_str());
-    m_prpt->WriteLine(sx_imported.c_str());
+    StringX sxImportedEntry;
+    // Use new group if the entries have been imported under a new level.
+    Format(sxImportedEntry, GROUPTITLEUSERINCHEVRONS,
+                        sxnewgroup.c_str(), cur_entry->title.c_str(),
+                        cur_entry->username.c_str());
+    m_prpt->WriteLine(sxImportedEntry.c_str());
 
     if (bNoPolicy) {
-      Format(sx_imported, IDSC_MISSINGPOLICYNAME, sxMissingPolicyName.c_str());
-      m_prpt->WriteLine(sx_imported.c_str());
+      Format(sxImportedEntry, IDSC_MISSINGPOLICYNAME, sxMissingPolicyName.c_str());
+      m_prpt->WriteLine(sxImportedEntry.c_str());
     }
 
+    // Need to check that entry keyboard shortcut not already in use!
+    int32 iKBShortcut;
+    ci_temp.GetKBShortcut(iKBShortcut);
+    
+    if (iKBShortcut != 0) {
+      // Check if already in use as an Entry Keyboard Shortcut
+      CUUID existingUUID = m_pXMLcore->GetKBShortcut(iKBShortcut);
+      if (existingUUID != CUUID::NullUUID()) {
+        // Remove it
+        ci_temp.SetKBShortcut(0);
+        ItemListIter iter = m_pXMLcore->Find(existingUUID);
+        if (iter == m_pXMLcore->GetEntryEndIter())
+          break;
+
+        // Tell the user via the report
+        StringX sxExistingEntry;
+        Format(sxExistingEntry, GROUPTITLEUSERINCHEVRONS,
+                           iter->second.GetGroup().c_str(), iter->second.GetTitle().c_str(),
+                           iter->second.GetUser().c_str());
+
+        StringX sxTemp, sxImported;
+        LoadAString(sxImported, IDSC_IMPORTED);
+        Format(sxTemp, IDSC_KBSHORTCUT_REMOVED,
+               sxImported.c_str(), sxImportedEntry.c_str(), sxExistingEntry.c_str(), sxImported.c_str());
+        m_prpt->WriteLine(sxTemp.c_str());
+        m_numShortcutsRemoved++;
+      }
+      // Check if already in use as an the PaswordSafe Application HotKey
+      if (m_pXMLcore->GetAppHotKey() == iKBShortcut) {
+        // Remove it
+        ci_temp.SetKBShortcut(0);
+
+        // Tell the user via the report
+        StringX sxTemp, sxImported;
+        LoadAString(sxImported, IDSC_IMPORTED);
+        Format(sxTemp, IDSC_KBSHORTCUT_USEBYAPP, sxImported.c_str(), sxImportedEntry.c_str());
+        m_prpt->WriteLine(sxTemp.c_str());
+        m_numShortcutsRemoved++;
+      }
+    }
     m_pXMLcore->GUISetupDisplayInfo(ci_temp);
     Command *pcmd = AddEntryCommand::Create(m_pXMLcore, ci_temp);
     pcmd->SetNoGUINotify();
@@ -907,12 +951,12 @@ void XMLFileHandlers::AddXMLEntries()
     delete cur_entry;
   }
 
-  Command *pcmdA = AddDependentEntriesCommand::Create(m_pXMLcore, *m_pPossible_Aliases, m_prpt, 
+  Command *pcmdA = AddDependentEntriesCommand::Create(m_pXMLcore, *m_pPossible_Aliases, m_prpt,
                                                       CItemData::ET_ALIAS,
                                                       CItemData::PASSWORD);
   pcmdA->SetNoGUINotify();
   m_pmulticmds->Add(pcmdA);
-  Command *pcmdS = AddDependentEntriesCommand::Create(m_pXMLcore, *m_pPossible_Shortcuts, m_prpt, 
+  Command *pcmdS = AddDependentEntriesCommand::Create(m_pXMLcore, *m_pPossible_Shortcuts, m_prpt,
                                                       CItemData::ET_SHORTCUT,
                                                       CItemData::PASSWORD);
   pcmdS->SetNoGUINotify();
@@ -922,7 +966,7 @@ void XMLFileHandlers::AddXMLEntries()
                                             UpdateGUICommand::WN_EXECUTE_REDO,
                                             UpdateGUICommand::GUI_REDO_IMPORT);
   m_pmulticmds->Add(pcmd2);
-  
+
   if (!sxEntriesWithNewNamedPolicies.empty()) {
     StringX sxRenamedPolicies;
     Format(sxRenamedPolicies, IDSC_ENTRIES_POLICIES, m_sxXML_DateTime.c_str(),
