@@ -9,7 +9,6 @@
 #pragma once
 
 #include "SecString.h"        // for CSecEditExtn
-#include "InfoDisplay.h"      // for Listbox Tooltips
 #include <vector>             // for Listbox Tooltips & EditExtn menus
 
 // ControlExtns.h : header file
@@ -224,6 +223,7 @@ private:
 };
 
 class CComboBoxExtn;
+class CInfoDisplay;
 
 class CListBoxExtn : public CListBox
 {
@@ -301,7 +301,8 @@ class CSymbolEdit : public CEdit
 {
   // Construction
 public:
-  CSymbolEdit() {}
+  CSymbolEdit();
+  void SetValidSym(const stringT &s);
 
 protected:
   //{{AFX_MSG(CSymbolEdit)
@@ -310,6 +311,8 @@ protected:
   //}}AFX_MSG
 
   DECLARE_MESSAGE_MAP()
+private:
+  stringT m_validSym; // defaults to CPasswordCharPool::GetDefaultSymbols()
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -348,3 +351,31 @@ private:
   int m_icolour;
   int m_type;
 };
+
+/////////////////////////////////////////////////////////////////////////////
+// CButtonBitmapExtn
+
+class CButtonBitmapExtn : public CButton
+{
+  // Construction
+public:
+  CButtonBitmapExtn();
+  virtual ~CButtonBitmapExtn();
+
+  void SetBitmapMaskAndID(const COLORREF cfMAsk, const int IDB)
+  {
+    m_cfMAsk = cfMAsk;  m_IDB = IDB;
+  }
+
+protected:
+  //{{AFX_MSG(CButtonBitmapExtn)
+  //}}AFX_MSG
+
+  DECLARE_MESSAGE_MAP()
+
+private:
+  void DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct);
+
+  int m_IDB;
+  COLORREF m_cfMAsk;
+};
