@@ -22,7 +22,6 @@ down the streetsky.  [Groucho Marx]
 #include "DboxMain.h" // for CheckPasskey()
 #include "PWSversion.h"
 #include "GeneralMsgBox.h"
-#include "SDThread.h"
 
 #include "core/PwsPlatform.h"
 #include "core/Pwsdirs.h"
@@ -34,6 +33,7 @@ down the streetsky.  [Groucho Marx]
 
 #include "os/file.h"
 #include "os/dir.h"
+#include "os/env.h"
 
 #include "VirtualKeyboard/VKeyBoardDlg.h"
 
@@ -47,31 +47,24 @@ down the streetsky.  [Groucho Marx]
 #include <iomanip>  // For setbase and setw
 
 // See DboxMain.h for the relevant enum
-int CPasskeyEntry::dialog_lookup[10] = {
-  // Normal dialogs
+int CPasskeyEntry::dialog_lookup[5] = {
   IDD_PASSKEYENTRY_FIRST,          // GCP_FIRST
   IDD_PASSKEYENTRY,                // GCP_NORMAL
   IDD_PASSKEYENTRY,                // GCP_RESTORE
   IDD_PASSKEYENTRY_WITHEXIT,       // GCP_WITHEXIT
   IDD_PASSKEYENTRY,                // GCP_CHANGEMODE
-  // Secure Desktop dialogs
-  IDD_PASSKEYENTRY_FIRST_SD,       // GCP_FIRST_SD
-  IDD_PASSKEYENTRY_SD,             // GCP_NORMAL_SD
-  IDD_PASSKEYENTRY_SD,             // GCP_RESTORE_SD
-  IDD_PASSKEYENTRY_WITHEXIT_SD,    // GCP_WITHEXIT_SD
-  IDD_PASSKEYENTRY_SD              // GCP_CHANGEMODE_SD
 };
 
 //-----------------------------------------------------------------------------
 CPasskeyEntry::CPasskeyEntry(CWnd* pParent, const CString& a_filespec, int index,
-                             bool bReadOnly, bool bForceReadOnly, bool bHideReadOnly,
-                             bool bUseSecureDesktop)
-  : CPKBaseDlg(dialog_lookup[index + (bUseSecureDesktop ? NUM_PER_ENVIRONMENT : 0)], pParent, bUseSecureDesktop),
+                             bool bReadOnly, bool bForceReadOnly, bool bHideReadOnly)
+  : CPKBaseDlg(dialog_lookup[index], pParent),
   m_filespec(a_filespec), m_orig_filespec(a_filespec),
   m_tries(0),
   m_status(TAR_INVALID),
   m_PKE_ReadOnly(bReadOnly ? TRUE : FALSE),
-  m_bForceReadOnly(bForceReadOnly), m_bHideReadOnly(bHideReadOnly),
+  m_bForceReadOnly(bForceReadOnly),
+    m_bHideReadOnly(bHideReadOnly),
   m_yubi_sk(NULL)
 {
   m_index = index;
@@ -119,11 +112,7 @@ void CPasskeyEntry::DoDataExchange(CDataExchange* pDX)
   DDX_Control(pDX, IDC_STATIC_LOGO, m_ctlLogo);
   DDX_Text(pDX, IDC_SELECTED_DATABASE, m_SelectedDatabase);
   DDX_Check(pDX, IDC_READONLY, m_PKE_ReadOnly);
-
-  if (!m_bUseSecureDesktop)
-    DDX_Control(pDX, IDOK, m_ctlOK);
-  else
-    DDX_Control(pDX, IDC_ENTERCOMBINATION, m_ctlEnterCombination);
+  DDX_Control(pDX, IDOK, m_ctlOK);
   //}}AFX_DATA_MAP
 }
 
@@ -136,11 +125,11 @@ BEGIN_MESSAGE_MAP(CPasskeyEntry, CPKBaseDlg)
   ON_BN_CLICKED(IDC_EXIT, OnExit)
   ON_CBN_EDITCHANGE(IDC_DATABASECOMBO, OnComboEditChange)
   ON_CBN_SELCHANGE(IDC_DATABASECOMBO, OnComboSelChange)
+  ON_BN_CLICKED(IDC_READONLY, OnBnClickedReadonly)
   ON_BN_CLICKED(IDC_BTN_BROWSE, OnOpenFileBrowser)
   ON_MESSAGE(PWS_MSG_INSERTBUFFER, OnInsertBuffer)
   ON_STN_CLICKED(IDC_VKB, OnVirtualKeyboard)
   ON_BN_CLICKED(IDC_YUBIKEY_BTN, OnYubikeyBtn)
-  ON_BN_CLICKED(IDC_ENTERCOMBINATION, OnEnterCombination)
   //}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
@@ -161,65 +150,49 @@ BOOL CPasskeyEntry::OnInitDialog(void)
 {
   CPKBaseDlg::OnInitDialog();
 
-  if (!m_bUseSecureDesktop) {
-    Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_PASSKEY));
+  Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_PASSKEY));
+  m_pctlPasskey->SetPasswordChar(PSSWDCHAR);
 
-    m_pctlPasskey->SetPasswordChar(PSSWDCHAR);
-  }
-
-  switch (m_index) {
-  case GCP_FIRST:
-    // At start up - give the user the option unless file is R-O
-    GetDlgItem(IDC_READONLY)->EnableWindow(m_bForceReadOnly ? FALSE : TRUE);
-    GetDlgItem(IDC_READONLY)->ShowWindow(SW_SHOW);
-    GetDlgItem(IDC_VERSION)->SetWindowText(m_appversion);
-#ifdef DEMO
-    GetDlgItem(IDC_SPCL_TXT)->
-      SetWindowText(CString(MAKEINTRESOURCE(IDS_DEMO)));
-#endif
-    break;
-  case GCP_NORMAL:
-    // otherwise during open - user can - again unless file is R-O
-    if (m_bHideReadOnly) {
-      GetDlgItem(IDC_READONLY)->EnableWindow(FALSE);
-      GetDlgItem(IDC_READONLY)->ShowWindow(SW_HIDE);
-    }
-    else {
+  switch(m_index) {
+    case GCP_FIRST:
+      // At start up - give the user the option unless file is R-O
       GetDlgItem(IDC_READONLY)->EnableWindow(m_bForceReadOnly ? FALSE : TRUE);
       GetDlgItem(IDC_READONLY)->ShowWindow(SW_SHOW);
-    }
-    break;
-  case GCP_RESTORE:
-  case GCP_WITHEXIT:
-    GetDlgItem(IDC_READONLY)->EnableWindow(m_bForceReadOnly ? FALSE : TRUE);
-    GetDlgItem(IDC_READONLY)->ShowWindow(SW_SHOW);
-    break;
-  case GCP_CHANGEMODE:
-    GetDlgItem(IDC_READONLY)->EnableWindow(FALSE);
-    GetDlgItem(IDC_READONLY)->ShowWindow(SW_HIDE);
-    break;
-  default:
-    ASSERT(FALSE);
+      GetDlgItem(IDC_VERSION)->SetWindowText(m_appversion);
+#ifdef DEMO
+      GetDlgItem(IDC_SPCL_TXT)->
+         SetWindowText(CString(MAKEINTRESOURCE(IDS_DEMO)));
+#endif
+      break;
+    case GCP_NORMAL:
+      // otherwise during open - user can - again unless file is R-O
+      if (m_bHideReadOnly) {
+        GetDlgItem(IDC_READONLY)->EnableWindow(FALSE);
+        GetDlgItem(IDC_READONLY)->ShowWindow(SW_HIDE);
+      } else {
+        GetDlgItem(IDC_READONLY)->EnableWindow(m_bForceReadOnly ? FALSE : TRUE);
+        GetDlgItem(IDC_READONLY)->ShowWindow(SW_SHOW);
+      }
+      break;
+    case GCP_RESTORE:
+    case GCP_WITHEXIT:
+      GetDlgItem(IDC_READONLY)->EnableWindow(m_bForceReadOnly ? FALSE : TRUE);
+      GetDlgItem(IDC_READONLY)->ShowWindow(SW_SHOW);
+      break;
+    case GCP_CHANGEMODE:
+      GetDlgItem(IDC_READONLY)->EnableWindow(FALSE);
+      GetDlgItem(IDC_READONLY)->ShowWindow(SW_HIDE);
+      break;
+    default:
+      ASSERT(FALSE);
   }
 
   // Only show virtual Keyboard menu if we can load DLL
-  if (!m_bUseSecureDesktop && !m_bVKAvailable) {
+  if (!CVKeyBoardDlg::IsOSKAvailable()) {
     GetDlgItem(IDC_VKB)->ShowWindow(SW_HIDE);
     GetDlgItem(IDC_VKB)->EnableWindow(FALSE);
   }
 
-  if (m_SelectedDatabase.IsEmpty()) {
-    if (m_index == GCP_FIRST) {
-      if (m_bUseSecureDesktop) {
-        m_pctlPasskey->EnableWindow(FALSE);
-        m_ctlOK.EnableWindow(TRUE);
-        m_SelectedDatabase.LoadString(IDS_NOCURRENTSAFE);
-      }
-      else
-        m_ctlEnterCombination.EnableWindow(FALSE);
-    }
-  }
-
   if (m_index == GCP_FIRST) {
     GetDlgItem(IDC_SELECTED_DATABASE)->ShowWindow(SW_HIDE);
 
@@ -270,7 +243,7 @@ BOOL CPasskeyEntry::OnInitDialog(void)
 
   if (m_index == GCP_FIRST) {
     m_ctlLogoText.ReloadBitmap(IDB_PSLOGO);
-    m_ctlLogo.ReloadBitmap(m_bUseSecureDesktop ? IDB_CLOGO_SMALL : IDB_CLOGO);
+    m_ctlLogo.ReloadBitmap(IDB_CLOGO);
   } else {
     m_ctlLogo.ReloadBitmap(IDB_CLOGO_SMALL);
   }
@@ -290,8 +263,7 @@ BOOL CPasskeyEntry::OnInitDialog(void)
     SetWindowPos(&CWnd::wndTopMost, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
     SetActiveWindow();
     SetForegroundWindow();
-    if (!m_bUseSecureDesktop)
-      m_pctlPasskey->SetFocus();
+    m_pctlPasskey->SetFocus();
     return FALSE;
   }
 
@@ -302,8 +274,7 @@ BOOL CPasskeyEntry::OnInitDialog(void)
   // a password:
   if (m_index == GCP_FIRST && !m_filespec.IsEmpty()) {
     m_MRU_combo.SetEditSel(-1, -1);
-    if (!m_bUseSecureDesktop)
-      m_pctlPasskey->SetFocus();
+    m_pctlPasskey->SetFocus();
     return FALSE;
   }
 
@@ -364,27 +335,18 @@ void CPasskeyEntry::OnCreateDb()
   }
 
   // 2. Get a password
-  bool bUseSecureDesktop = PWSprefs::GetInstance()->GetPref(PWSprefs::UseSecureDesktop);
 
-  do
-  {
-    CPasskeySetup pksetup(this, *app.GetCore(), bUseSecureDesktop);
-    rc = pksetup.DoModal();
+  CPasskeySetup pksetup(this, *app.GetCore());
+  rc = pksetup.DoModal();
 
-    if (rc == IDOK)
-      m_passkey = pksetup.GetPassKey();
-
-    // In case user wanted to toggle Secure Desktop
-    bUseSecureDesktop = !bUseSecureDesktop;
-  } while (rc == INT_MAX);
-
-  if (rc != IDOK)
+  if (rc == IDOK)
+    m_passkey = pksetup.GetPassKey();
+  else
     return;  //User cancelled password entry
 
   // 3. Set m_filespec && m_passkey to returned value!
   m_filespec = newfile;
-  if (!m_bUseSecureDesktop)
-    ((CEdit*)GetDlgItem(IDC_PASSKEY))->SetWindowText(m_passkey);
+  ((CEdit*)GetDlgItem(IDC_PASSKEY))->SetWindowText(m_passkey);
 
   m_status = TAR_NEW;
   CPWDialog::OnOK();
@@ -460,10 +422,8 @@ void CPasskeyEntry::ProcessPhrase()
     else { // try again
       gmb.AfxMessageBox(m_index == GCP_CHANGEMODE ? IDS_BADPASSKEY : IDS_INCORRECTKEY);
     }
-    if (!m_bUseSecureDesktop) {
-      m_pctlPasskey->SetSel(MAKEWORD(-1, 0));
-      m_pctlPasskey->SetFocus();
-    }
+    m_pctlPasskey->SetSel(MAKEWORD(-1, 0));
+    m_pctlPasskey->SetFocus();
     break;
   case PWScore::READ_FAIL:
     gmb.AfxMessageBox(IDSC_FILE_UNREADABLE);
@@ -490,20 +450,17 @@ void CPasskeyEntry::OnHelp()
 
 void CPasskeyEntry::UpdateRO()
 {
-  if (!m_bForceReadOnly) { // if allowed, changed r-o state to reflect file's permission
+  if (!m_bForceReadOnly) {
+    // If allowed, change R-O state to reflect file's permission - only if file is R-O
     bool fro;
     if (pws_os::FileExists(LPCWSTR(m_filespec), fro) && fro) {
       m_PKE_ReadOnly = TRUE;
       GetDlgItem(IDC_READONLY)->EnableWindow(FALSE);
-    } else { // no file or write-enabled
-      if (m_index == GCP_FIRST)
-        m_PKE_ReadOnly = PWSprefs::GetInstance()->GetPref(PWSprefs::DefaultOpenRO) ? TRUE : FALSE;
-      else
-        m_PKE_ReadOnly = FALSE;
+      ((CButton *)GetDlgItem(IDC_READONLY))->SetCheck(BST_CHECKED);
+    } else {
       GetDlgItem(IDC_READONLY)->EnableWindow(TRUE);
     }
-    ((CButton *)GetDlgItem(IDC_READONLY))->SetCheck(m_PKE_ReadOnly == TRUE ? 
-                                                    BST_CHECKED : BST_UNCHECKED);
+
     UpdateData(FALSE);
   } // !m_bForceReadOnly
 }
@@ -511,18 +468,15 @@ void CPasskeyEntry::UpdateRO()
 void CPasskeyEntry::OnComboEditChange()
 {
   m_MRU_combo.m_edit.GetWindowText(m_filespec);
-  if (!m_bUseSecureDesktop) {
-    m_pctlPasskey->EnableWindow(TRUE);
-    m_ctlOK.EnableWindow(TRUE);
-  }
-  else
-  {
-    m_ctlEnterCombination.EnableWindow(TRUE);
-  }
-
   UpdateRO();
 }
 
+
+void CPasskeyEntry::OnBnClickedReadonly()
+{
+  m_PKE_ReadOnly = ((CButton *)GetDlgItem(IDC_READONLY))->GetCheck() == BST_CHECKED;
+}
+
 void CPasskeyEntry::OnComboSelChange()
 {
   CRecentFileList *mru = app.GetMRU();
@@ -543,20 +497,7 @@ void CPasskeyEntry::OnComboSelChange()
       m_filespec = m_orig_filespec;
   }
 
-  if (!m_bUseSecureDesktop) {
-    m_pctlPasskey->EnableWindow(TRUE);
-    m_pctlPasskey->SetFocus();
-    m_ctlOK.EnableWindow(TRUE);
-  }
-  else
-  {
-    CString csText;
-    csText.LoadString(m_filespec.IsEmpty() ? IDS_OK : IDS_ENTERCOMBINATION);
-
-    m_ctlEnterCombination.SetWindowText(csText);
-    m_ctlEnterCombination.EnableWindow(TRUE);
-  }
-
+  m_pctlPasskey->SetFocus();
   UpdateRO();
 }
 
@@ -574,10 +515,13 @@ void CPasskeyEntry::OnOpenFileBrowser()
 
   fd.m_ofn.lpstrTitle = cs_text;
 
-  if (PWSprefs::GetInstance()->GetPref(PWSprefs::DefaultOpenRO))
+  if (!pws_os::IsWindowsVistaOrGreater()) {
+    // Read-only checkbox only available up to Windows XP
+    if (PWSprefs::GetInstance()->GetPref(PWSprefs::DefaultOpenRO))
       fd.m_ofn.Flags |= OFN_READONLY;
     else
       fd.m_ofn.Flags &= ~OFN_READONLY;
+  }
 
   std::wstring dir;
   if (GetMainDlg()->GetCurFile().empty())
@@ -601,21 +545,16 @@ void CPasskeyEntry::OnOpenFileBrowser()
     return;
   }
   if (rc == IDOK) {
-    m_PKE_ReadOnly = fd.GetReadOnlyPref();
+    if (!pws_os::IsWindowsVistaOrGreater()) {
+      // Read-only checkbox only available up to Windows XP
+      // In XP, the checkbox setting overrides the main dialog checkbox
+      // but the user can then change it in the main dialog if they want
+      m_PKE_ReadOnly = fd.GetReadOnlyPref();
+    }
+
     m_filespec = fd.GetPathName();
     m_MRU_combo.m_edit.SetWindowText(m_filespec);
-    if (!m_bUseSecureDesktop) {
-      // Normal
-      m_pctlPasskey->EnableWindow(TRUE);
-      if (m_pctlPasskey->IsWindowEnabled() == TRUE) {
-        m_pctlPasskey->SetFocus();
-      }
-      m_ctlOK.EnableWindow(TRUE);
-    }
-    else {
-      // Secure Desktop
-      m_ctlEnterCombination.EnableWindow(TRUE);
-    }
+    m_pctlPasskey->SetFocus();
     UpdateRO();
   } // rc == IDOK
 }
@@ -648,43 +587,31 @@ void CPasskeyEntry::SetHeight(const int num)
 
 void CPasskeyEntry::OnVirtualKeyboard()
 {
-  // This is used if Secure Desktop isn't!
-  DWORD dwError; //  Define it here to stop warning that local variable is initialized but not referenced later on
-
   // Shouldn't be here if couldn't load DLL. Static control disabled/hidden
-  if (!m_bVKAvailable)
+  if (!CVKeyBoardDlg::IsOSKAvailable())
     return;
 
-  if (m_hwndVKeyBoard != NULL && ::IsWindowVisible(m_hwndVKeyBoard)) {
+  if (m_pVKeyBoardDlg != NULL && m_pVKeyBoardDlg->IsWindowVisible()) {
     // Already there - move to top
-    ::SetWindowPos(m_hwndVKeyBoard, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
+    m_pVKeyBoardDlg->SetWindowPos(&wndTop , 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
     return;
   }
 
   // If not already created - do it, otherwise just reset it
   if (m_pVKeyBoardDlg == NULL) {
     StringX cs_LUKBD = PWSprefs::GetInstance()->GetPref(PWSprefs::LastUsedKeyboard);
-    m_pVKeyBoardDlg = new CVKeyBoardDlg(this->GetSafeHwnd(), this->GetSafeHwnd(), cs_LUKBD.c_str());
-    m_hwndVKeyBoard = CreateDialogParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_SDVKEYBOARD), this->GetSafeHwnd(),
-      (DLGPROC)(m_pVKeyBoardDlg->VKDialogProc), (LPARAM)(m_pVKeyBoardDlg));
-
-    if (m_hwndVKeyBoard == NULL) {
-      dwError = pws_os::IssueError(_T("CreateDialogParam - IDD_SDVKEYBOARD"), false);
-      ASSERT(m_hwndVKeyBoard);
-    }
+    m_pVKeyBoardDlg = new CVKeyBoardDlg(this, cs_LUKBD.c_str());
+    m_pVKeyBoardDlg->Create(CVKeyBoardDlg::IDD);
   } else {
     m_pVKeyBoardDlg->ResetKeyboard();
   }
 
-  // Now show it and make it top & enable it
-  ::SetWindowPos(m_hwndVKeyBoard, HWND_TOP, 0, 0, 0, 0, SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);
-  ::EnableWindow(m_hwndVKeyBoard, TRUE);
+  // Now show it and make it top
+  m_pVKeyBoardDlg->SetWindowPos(&wndTop , 0, 0, 0, 0, SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);
 }
 
 LRESULT CPasskeyEntry::OnInsertBuffer(WPARAM, LPARAM)
 {
-  // This is used if Secure Desktop isn't!
-
   // Update the variables
   UpdateData(TRUE);
 
@@ -724,27 +651,3 @@ void CPasskeyEntry::OnYubikeyBtn()
   }
   yubiRequestHMACSha1(m_passkey);
 }
-
-void CPasskeyEntry::OnEnterCombination()
-{
-  if (m_filespec.IsEmpty()) {
-    m_status = TAR_OPEN_NODB;
-    CPWDialog::OnCancel();
-    return;
-  }
-
-  // Get passphrase from Secure Desktop
-  StartThread(IDD_SDGETPHRASE);
-
-  ShowWindow(SW_SHOW);
-
-  if (m_GMP.bPhraseEntered) {
-    ShowWindow(SW_SHOW);
-    m_passkey = m_GMP.sPhrase.c_str();
-    ProcessPhrase();
-  }
-
-  // Just do nothing if no passphrase entered i.e. user pressed cancel
-  // Try to get this seen
-  SetForegroundWindow();
-}
