@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -103,11 +103,6 @@ CSystemTray::CSystemTray(CWnd* pParent, UINT uCallbackMessage, LPCWSTR szToolTip
 BOOL CSystemTray::Create(CWnd *pParent, UINT uCallbackMessage, LPCWSTR szToolTip,
                          HICON icon, UINT uID, UINT menuID)
 {
-  // this is only for Windows 95 (or higher)
-  m_bEnabled = (GetVersion() & 0xff) >= 4;
-  if (!m_bEnabled)
-    return FALSE;
-
   // Make sure Notification window is valid (not needed - CJM)
   // VERIFY(m_bEnabled = (pParent && ::IsWindow(pParent->GetSafeHwnd())));
   // if (!m_bEnabled) return FALSE;
@@ -129,11 +124,7 @@ BOOL CSystemTray::Create(CWnd *pParent, UINT uCallbackMessage, LPCWSTR szToolTip
   m_tnd.hIcon  = icon;
   m_tnd.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
   m_tnd.uCallbackMessage = uCallbackMessage;
-#if (_MSC_VER >= 1400)
   wcsncpy_s(m_tnd.szTip, sizeof(m_tnd.szTip), ttt.c_str(), ttt.length());
-#else
-  wcsncpy(m_tnd.szTip, ttt.c_str(), ttt.length());
-#endif
 
   // Set the tray icon
   m_bEnabled = Shell_NotifyIcon(NIM_ADD, &m_tnd);
@@ -150,13 +141,6 @@ CSystemTray::~CSystemTray()
   ::DestroyIcon(m_hSavedIcon);
 }
 
-CWnd *CSystemTray::SetTarget(CWnd *pTarget)
-{
-  CWnd *pOldTarget = m_pTarget;
-  m_pTarget = pTarget;
-  return pOldTarget;
-}
-
 /////////////////////////////////////////////////////////////////////////////
 // CSystemTray icon manipulation
 
@@ -326,11 +310,7 @@ BOOL CSystemTray::SetTooltipText(LPCWSTR pszTip)
 
   StringX ttt = PWSUtil::NormalizeTTT(pszTip, MAX_TTT_LEN);
   m_tnd.uFlags = NIF_TIP;
-#if (_MSC_VER >= 1400)
   wcsncpy_s(m_tnd.szTip, sizeof(m_tnd.szTip), ttt.c_str(), ttt.length());
-#else
-  wcsncpy(m_tnd.szTip, ttt.c_str(), ttt.length());
-#endif
 
   return Shell_NotifyIcon(NIM_MODIFY, &m_tnd);
 }
@@ -402,8 +382,7 @@ void CSystemTray::OnTimer(UINT_PTR )
 }
 
 // Helper function to set up recent entry submenu based on entry's attributes
-static BOOL SetupRecentEntryMenu(CMenu *&pMenu, const int i, const CItemData *pci,
-                                 const DboxMain *pDbx)
+static BOOL SetupRecentEntryMenu(CMenu *&pMenu, const int i, const CItemData *pci)
 {
   BOOL brc;
   CString cs_text, cs_select;
@@ -431,10 +410,10 @@ static BOOL SetupRecentEntryMenu(CMenu *&pMenu, const int i, const CItemData *pc
   if (brc == 0) goto exit;
   ipos++;
 
-  if (pci->IsShortcut() && pDbx != NULL) {
+  if (pci->IsShortcut()) {
     // Shortcut has no data of itself - for all other menu items
     // use base entry's fields
-    const CItemData *pBase = pDbx->GetBaseEntry(pci);
+    const CItemData *pBase = app.GetMainDlg()->GetBaseEntry(pci);
     if (pBase != NULL)
       pci = pBase;
   }
@@ -630,13 +609,11 @@ LRESULT CSystemTray::OnTrayNotification(WPARAM wParam, LPARAM lParam)
 
       // No point in doing Recent Entries if database is locked
       if (num_recent_entries != 0 && app_state == ThisMfcApp::UNLOCKED) {
-        DboxMain *pDbx = dynamic_cast<DboxMain *>(m_pParent);
-        ASSERT(pDbx != NULL); // fail safely in release build later
         // Build extra popup menus (1 per entry in list)
         typedef CMenu* CMenuPtr;
         ppNewRecentEntryMenu = new CMenuPtr[num_recent_entries];
         m_RUEList.GetAllMenuItemStrings(m_menulist);
-        const bool bGUIEmpty = pDbx == NULL || pDbx->IsGUIEmpty();
+        const bool bGUIEmpty = app.GetMainDlg()->IsGUIEmpty();
 
         for (size_t i = 0; i < num_recent_entries; i++) {
           ppNewRecentEntryMenu[i] = NULL;  // Ensure empty
@@ -648,7 +625,7 @@ LRESULT CSystemTray::OnTrayNotification(WPARAM wParam, LPARAM lParam)
             continue;
           }
 
-          BOOL brc = SetupRecentEntryMenu(ppNewRecentEntryMenu[i], (int)i, pci, pDbx);
+          BOOL brc = SetupRecentEntryMenu(ppNewRecentEntryMenu[i], (int)i, pci);
           if (brc == 0) {
             pws_os::Trace(L"CSystemTray::OnTrayNotification: SetupRecentEntryMenu - ppNewRecentEntryMenu[%d] failed\n", i);
             continue;
