@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
+ * Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -7,7 +7,7 @@
  */
 
 /** \file PwsSync.cpp
-* 
+*
 */
 
 
@@ -28,6 +28,7 @@
 #include "./DbSelectionPanel.h"
 #include "./AdvancedSelectionDlg.h"
 #include "../../core/PWScore.h"
+#include "./SelectionCriteria.h"
 
 #include <wx/filename.h>
 #include <wx/valgen.h>
@@ -42,7 +43,7 @@
 
 /*!
  * SyncData class declaration
- * 
+ *
  * Sync data and options shared and updated by all pages
  */
 struct SyncData {
@@ -57,7 +58,7 @@ struct SyncData {
 
 /*!
  * SyncWizardPage class declaration
- * 
+ *
  * Base class from which all other Sync wizard pages are derived.  Each page
  * must overload the "SaveData" function
  */
@@ -67,7 +68,7 @@ protected:
   SyncData* m_syncData;
   typedef enum {BACKWARD, FORWARD} PageDirection;
   wxBoxSizer* m_pageSizer;
-  
+
 public:
   SyncWizardPage(wxWizard* parent, SyncData* data, const wxString& pageHeader);
   virtual void SaveData(SyncData* data) = 0;
@@ -86,8 +87,8 @@ public:
 
 /*!
  * SyncStartPage class declaration
- * 
- * First page of the synchronization wizard.  Only displays a 
+ *
+ * First page of the synchronization wizard.  Only displays a
  * welcome message and explains the functionality
  */
 class SyncStartPage : public SyncWizardPage
@@ -100,7 +101,7 @@ public:
 
 /*!
  * DbSelectionPage class declaration
- * 
+ *
  * Second page of the synchronization wizard.  Lets the user chose
  * the DB to synchronize with and enter its combination
  */
@@ -123,7 +124,22 @@ struct SyncFieldSelection {
   static bool IsMandatoryField(CItemData::FieldType /*field*/) {
     return false;
   }
-  
+
+  static bool IsPreselectedField(CItemData::FieldType /*field*/) {
+    return true;
+  }
+
+  static bool IsUsableField(CItemData::FieldType field) {
+    switch (field) {
+      case CItemData::GROUP:
+      case CItemData::USER:
+      case CItemData::TITLE:
+        return false;
+      default:
+        return true;
+    }
+  }
+
   static bool ShowFieldSelection() {
     return true;
   }
@@ -134,7 +150,7 @@ struct SyncFieldSelection {
 
 /*!
  * SyncFieldSelectionPage class declaration
- * 
+ *
  * Third page of the synchronization wizard.  Lets the user chose
  * which fields to synchronize
  */
@@ -153,7 +169,7 @@ public:
 
 /*!
  * SyncOptionsSummaryPage class declaration
- * 
+ *
  * Fourth page of the synchronization wizard.  Presents the summary of selected
  * options in a language the user would understand
  */
@@ -165,14 +181,14 @@ class SyncOptionsSummaryPage: public SyncWizardPage
 
 public:
   SyncOptionsSummaryPage(wxWizard* parent, SyncData* data);
-  
+
   virtual void OnPageEnter(PageDirection dir);
   virtual void SaveData(SyncData* /*data*/) {}
 };
 
 /*!
  * SyncStatusPage class declaration
- * 
+ *
  * Final page of the synchronization wizard, where all the synchronization happens
  * Either shows a progressbar followed by a success/error message, or an error
  * if the sync couldn't even be started
@@ -180,7 +196,7 @@ public:
 class SyncStatusPage: public SyncWizardPage
 {
   enum {ID_HEADER_TXT = 100, ID_PROGRESS_TXT, ID_GAUGE, ID_FINISH_TXT, ID_SHOW_REPORT};
-  
+
   void SetSyncSummary(const wxString& str);
   void SetProgressText(const wxString& str);
   void SetHeaderText(const wxString& str);
@@ -192,7 +208,7 @@ class SyncStatusPage: public SyncWizardPage
 
 public:
   SyncStatusPage(wxWizard* parent, SyncData* data);
-  
+
   virtual void SaveData(SyncData* /*data*/) {}
   virtual void OnPageEnter(PageDirection dir);
 };
@@ -205,7 +221,7 @@ BEGIN_EVENT_TABLE(PwsSyncWizard, wxWizard)
   EVT_WIZARD_PAGE_CHANGING(wxID_ANY, PwsSyncWizard::OnWizardPageChanging)
 END_EVENT_TABLE()
 
-PwsSyncWizard::PwsSyncWizard(wxWindow* parent, PWScore* core): 
+PwsSyncWizard::PwsSyncWizard(wxWindow* parent, PWScore* core):
                 wxWizard(parent, wxID_ANY, _("Synchronize another database with currently open database")),
                 m_page1(0), m_syncData(new SyncData)
 {
@@ -225,21 +241,21 @@ PwsSyncWizard::PwsSyncWizard(wxWindow* parent, PWScore* core):
   m_syncData->showReport = false;
 
   m_page1 = new SyncStartPage(this, m_syncData);
-  
+
   DbSelectionPage*        page2 = new DbSelectionPage(this, m_syncData);
   SyncFieldSelectionPage* page3 = new SyncFieldSelectionPage(this, m_syncData);
   SyncOptionsSummaryPage* page4 = new SyncOptionsSummaryPage(this, m_syncData);
   SyncStatusPage*         page5 = new SyncStatusPage(this, m_syncData);
-  
+
   m_page1->SetNext(page2);
   page2->SetPrev(m_page1);
 
   page3->SetPrev(page2);
   page2->SetNext(page3);
-  
+
   page3->SetNext(page4);
   page4->SetPrev(page3);
-  
+
   page4->SetNext(page5);
 
   GetPageAreaSizer()->Add(m_page1);
@@ -286,7 +302,7 @@ BEGIN_EVENT_TABLE(SyncWizardPage, wxWizardPageSimple)
   EVT_WIZARD_PAGE_CHANGED( wxID_ANY, SyncWizardPage::OnWizardPageChanged)
 END_EVENT_TABLE()
 
-SyncWizardPage::SyncWizardPage(wxWizard* parent, SyncData* data, 
+SyncWizardPage::SyncWizardPage(wxWizard* parent, SyncData* data,
                                   const wxString& pageHeader): wxWizardPageSimple(parent),
                                                                m_syncData(data),
                                                                m_pageSizer(new wxBoxSizer(wxVERTICAL))
@@ -308,7 +324,7 @@ void SyncWizardPage::OnWizardPageChanging(wxWizardEvent& evt)
 {
   SyncWizardPage* page = wxDynamicCast(evt.GetPage(), SyncWizardPage);
   wxASSERT_MSG(page, wxT("Sync wizard page not derived from SyncWizardPage class"));
-  
+
   if (!page->OnPageLeave(evt.GetDirection()? FORWARD: BACKWARD))
     evt.Veto();
 
@@ -320,7 +336,7 @@ void SyncWizardPage::OnWizardPageChanged(wxWizardEvent& evt)
 {
   SyncWizardPage* page = wxDynamicCast(evt.GetPage(), SyncWizardPage);
   wxASSERT_MSG(page, wxT("Sync wizard page not derived from SyncWizardPage class"));
-  
+
   page->OnPageEnter(evt.GetDirection()? FORWARD: BACKWARD);
 
   //must always do this, to let the wizard see the event as well
@@ -344,14 +360,14 @@ SyncStartPage::SyncStartPage(wxWizard* parent, SyncData* data) : SyncWizardPage(
   sizer->Add(new wxStaticText(this, wxID_ANY, explanation), wxSizerFlags().Expand().Proportion(0).Border());
 
   wxCollapsiblePane* pane = new wxCollapsiblePane(this, wxID_ANY, _("More Info"));
-  
-  const wxChar* helpItems[] = {
+
+  const wxString helpItems[] = {
     _("1. Two entries from different databases match if their Group, Title\nand User fields match."),
     _("2. You can select the fields to update, as well as filter the entries\nfor synchronization."),
     _("3. Only existing entries in your database are updated.  No new entries are\nadded or existing entries removed during this process."),
     _("4. You can undo the operation once it is complete, but won't be\nable to abort it mid-way.")
   };
-  
+
   wxBoxSizer* paneSizer = new wxBoxSizer(wxVERTICAL);
   for (size_t idx = 0; idx < NumberOf(helpItems); ++idx) {
     paneSizer->Add(new wxStaticText(pane->GetPane(), wxID_ANY, helpItems[idx]), wxSizerFlags().Expand().Border().Proportion(1));
@@ -368,7 +384,7 @@ SyncStartPage::SyncStartPage(wxWizard* parent, SyncData* data) : SyncWizardPage(
 DbSelectionPage::DbSelectionPage(wxWizard* parent, SyncData* data):
                              SyncWizardPage(parent, data, _("Select another database"))
 {
-  const wxString filePrompt(wxString(_("Choose Database to Synchronize with \"")) << towxstring(data->core->GetCurFile()) << _("\""));
+  const wxString filePrompt(wxString(_("Choose Database to Synchronize with \"")) << towxstring(data->core->GetCurFile()) << wxT("\""));
   const wxString filePickerCtrlTitle(_("Please Choose a Database to Synchronize with current database"));
 
   wxBoxSizer* sizer = m_pageSizer;
@@ -395,8 +411,8 @@ SyncFieldSelectionPage::SyncFieldSelectionPage(wxWizard* parent, SyncData* data)
                                SyncWizardPage(parent, data, _("Synchronization options"))
 {
   wxBoxSizer* sizer = m_pageSizer;
-  
-  m_panel = new SyncFieldSelectionPanel(this, data->selCriteria, false);
+
+  m_panel = new SyncFieldSelectionPanel(this, &data->selCriteria, false);
   m_panel->CreateControls(this);
   sizer->Add(m_panel, wxSizerFlags().Expand().Proportion(1));
   SetSizerAndFit(sizer);
@@ -409,7 +425,7 @@ bool SyncFieldSelectionPage::OnPageLeave(PageDirection dir)
 
 void SyncFieldSelectionPage::SaveData(SyncData* data)
 {
-  data->selCriteria = m_panel->m_criteria;
+  data->selCriteria = *m_panel->m_criteria;
 }
 
 
@@ -424,7 +440,7 @@ SyncOptionsSummaryPage::SyncOptionsSummaryPage(wxWizard* parent, SyncData* data)
   wxSizerFlags flags = wxSizerFlags().Expand().Proportion(0).Border(wxLEFT+wxRIGHT, SideMargin);
   wxSizerFlags gridFlags = wxSizerFlags().Expand().Proportion(1).Border(wxLEFT+wxRIGHT, SideMargin*2);
   wxBoxSizer* sizer = m_pageSizer;
-  
+
   sizer->Add(new wxStaticText(this, ID_DESC, wxEmptyString), flags.Proportion(1));
   sizer->AddSpacer(RowSeparation);
 
@@ -433,7 +449,7 @@ SyncOptionsSummaryPage::SyncOptionsSummaryPage(wxWizard* parent, SyncData* data)
 
   m_updatedFieldsGrid = new wxFlexGridSizer(0, 3, RowSeparation, ColSeparation);
   sizer->Add(m_updatedFieldsGrid, gridFlags.Proportion(1));
-  sizer->AddSpacer(RowSeparation);
+  sizer->AddSpacer(RowSeparation*2);
 
   sizer->Add(new wxStaticText(this, ID_NOT_UPDATED_TXT, wxEmptyString), flags.Proportion(0));
   sizer->AddSpacer(RowSeparation);
@@ -446,7 +462,7 @@ SyncOptionsSummaryPage::SyncOptionsSummaryPage(wxWizard* parent, SyncData* data)
   wxStaticText* txtWarn = new wxStaticText(this, wxID_ANY, warning);
   txtWarn->SetForegroundColour(*wxRED);
   sizer->Add(txtWarn, flags.Proportion(0).Bottom());
-  
+
   SetSizerAndFit(sizer);
 }
 
@@ -457,7 +473,7 @@ void SyncOptionsSummaryPage::OnPageEnter(PageDirection dir)
 
   m_updatedFieldsGrid->Clear(true);
   m_notUpdatedFieldsGrid->Clear(true);
-  
+
   wxString description = m_syncData->selCriteria.GetGroupSelectionDescription();
   description << _(" will be updated with corresponding entries from \"")
               << m_syncData->otherDB.GetFullPath() << wxT('"');
@@ -480,6 +496,11 @@ void SyncOptionsSummaryPage::OnPageEnter(PageDirection dir)
     for( size_t idx = 0; idx < fieldsNotSelected.Count(); ++idx) {
       m_notUpdatedFieldsGrid->Add(new wxStaticText(this, wxID_ANY, wxT("* ") + fieldsNotSelected[idx]));
     }
+
+    // Set the wxSizer Proportion of the two grids of field lists to the number
+    // of rows they have now, or else they overlap each other
+    m_pageSizer->GetItem(m_updatedFieldsGrid)->SetProportion(m_updatedFieldsGrid->GetRows());
+    m_pageSizer->GetItem(m_notUpdatedFieldsGrid)->SetProportion(m_notUpdatedFieldsGrid->GetRows());
   }
   GetSizer()->Layout();
 }
@@ -538,7 +559,7 @@ void SyncStatusPage::OnPageEnter(PageDirection dir)
 
     FindWindow(ID_SHOW_REPORT)->Hide();
 
-    PWScore* othercore = new PWScore;
+    PWSAuxCore* othercore = new PWSAuxCore;
     const wxString otherDBPath = m_syncData->otherDB.GetFullPath();
     const int rc = ReadCore(*othercore, otherDBPath, m_syncData->combination,
                                     false, this);
@@ -552,13 +573,13 @@ void SyncStatusPage::OnPageEnter(PageDirection dir)
       }
     }
     else {
-      SetProgressText(wxString::Format(static_cast<const wxChar*>(GetReadErrorMessageTemplate(rc)),
-                        static_cast<const wxChar*>(otherDBPath)));
+      SetProgressText(wxString::Format(GetReadErrorMessageTemplate(rc),
+                                       otherDBPath));
       SetHeaderText(_("There was an error during synchronization"));
       SetSyncSummary(_("File Read Error"));
     }
   }
-  
+
   GetSizer()->Layout();
 }
 
@@ -567,9 +588,9 @@ void SyncStatusPage::OnSyncStartEvent(wxCommandEvent& evt)
   PWScore* otherCore = reinterpret_cast<PWScore*>(evt.GetClientData());
   wxASSERT_MSG(otherCore, wxT("Sync Start Event did not arrive with the other PWScore"));
   Synchronize(m_syncData->core, otherCore);
-  
+
   SetHeaderText(wxString::Format(_("Your database has been synchronized with \"%s\""), otherCore->GetCurFile().c_str()));
-  SetProgressText(wxString::Format(_("%d %s updated"), m_syncData->numUpdated, 
+  SetProgressText(wxString::Format(_("%d %s updated"), m_syncData->numUpdated,
                       m_syncData->numUpdated == 1? _("entry"): _("entries")));
   SetSyncSummary(_("Synchronization completed successfully"));
 
@@ -629,9 +650,9 @@ void SyncStatusPage::Synchronize(PWScore* currentCore, const PWScore *otherCore)
 {
   CReport& rpt = m_syncData->syncReport;
 
-  rpt.StartReport(_("Synchronize"), currentCore->GetCurFile().c_str());
-  wxString line = wxString::Format(_("Synchronizing from database: %s\n"), otherCore->GetCurFile().c_str());
-  rpt.WriteLine(static_cast<const wxChar*>(line));
+  rpt.StartReport(_("Synchronize").c_str(), currentCore->GetCurFile().c_str());
+  wxString line = wxString::Format(_("Synchronizing from database: %s\n").c_str(), otherCore->GetCurFile().c_str());
+  rpt.WriteLine(line.c_str());
 
   ReportAdvancedOptions(&rpt, _("synchronized"));
 
@@ -644,7 +665,7 @@ void SyncStatusPage::Synchronize(PWScore* currentCore, const PWScore *otherCore)
   pmulticmds->Add(pcmd1);
   const SelectionCriteria& criteria = m_syncData->selCriteria;
   const stringT subgroup_name = tostdstring(criteria.SubgroupSearchText());
-  
+
   wxGauge* gauge = wxDynamicCast(FindWindow(ID_GAUGE), wxGauge);
   gauge->SetRange(int(otherCore->GetNumEntries()));
 
@@ -666,17 +687,17 @@ void SyncStatusPage::Synchronize(PWScore* currentCore, const PWScore *otherCore)
     const StringX otherTitle = otherItem.GetTitle();
     const StringX otherUser = otherItem.GetUser();
 
-    const StringX sx_updated = StringX(L"\xab") + 
-                           otherGroup + StringX(L"\xbb \xab") + 
-                           otherTitle + StringX(L"\xbb \xab") +
-                           otherUser  + StringX(L"\xbb");
+    const StringX sx_updated = StringX(wxT("\xab")) +
+                           otherGroup + StringX(wxT("\xbb \xab")) +
+                           otherTitle + StringX(wxT("\xbb \xab")) +
+                           otherUser  + StringX(wxT("\xbb"));
 
     SetProgressText((wxString() << currentIndex << wxT(": ")) + towxstring(sx_updated));
     wxSafeYield();
 
-    if (criteria.HasSubgroupRestriction() && !otherItem.Matches(subgroup_name, criteria.SubgroupObject(), criteria.SubgroupFunctionWithCase())) 
+    if (criteria.HasSubgroupRestriction() && !otherItem.Matches(subgroup_name, criteria.SubgroupObject(), criteria.SubgroupFunctionWithCase()))
       continue;
-    
+
     ItemListConstIter foundPos = currentCore->Find(otherGroup, otherTitle, otherUser);
 
     if (foundPos != currentCore->GetEntryEndIter()) {
@@ -689,7 +710,7 @@ void SyncStatusPage::Synchronize(PWScore* currentCore, const PWScore *otherCore)
       curItem.GetUUID(current_uuid);
       otherItem.GetUUID(other_uuid);
       if (memcmp((void *)current_uuid, (void *)other_uuid, sizeof(uuid_array_t)) != 0) {
-        pws_os::Trace(L"Synchronize: Mis-match UUIDs for [%s:%s:%s]\n", otherGroup.c_str(), otherTitle.c_str(), otherUser.c_str());
+        pws_os::Trace(wxT("Synchronize: Mis-match UUIDs for [%s:%s:%s]\n"), otherGroup.c_str(), otherTitle.c_str(), otherUser.c_str());
       }
 
       bool bUpdated(false);
@@ -721,14 +742,14 @@ void SyncStatusPage::Synchronize(PWScore* currentCore, const PWScore *otherCore)
 
   if (numUpdated > 0) {
     std::sort(vs_updated.begin(), vs_updated.end(), MergeSyncGTUCompare);
-    const wxChar* cs_singular_plural_type = (numUpdated == 1 ? _("entry") : _("entries"));
-    const wxChar* cs_singular_plural_verb = (numUpdated == 1 ? _("was") : _("were"));
-    const wxString resultStr = wxString::Format(_("\nThe following %s %s updated:"), cs_singular_plural_type, 
-                                                                cs_singular_plural_verb);
-    rpt.WriteLine(static_cast<const wxChar*>(resultStr));
+    const wxString cs_singular_plural_type = (numUpdated == 1 ? _("entry") : _("entries"));
+    const wxString cs_singular_plural_verb = (numUpdated == 1 ? _("was") : _("were"));
+    const wxString resultStr = wxString::Format(_("\nThe following %s %s updated:").c_str(), cs_singular_plural_type,
+                                                cs_singular_plural_verb);
+    rpt.WriteLine(resultStr.c_str());
     for (size_t i = 0; i < vs_updated.size(); i++) {
       const wxString fieldName = wxString::Format(wxT("\t%s"), vs_updated[i].c_str());
-      rpt.WriteLine(static_cast<const wxChar*>(fieldName));
+      rpt.WriteLine(fieldName.c_str());
     }
   }
 
@@ -736,11 +757,11 @@ void SyncStatusPage::Synchronize(PWScore* currentCore, const PWScore *otherCore)
                                             UpdateGUICommand::GUI_REDO_MERGESYNC);
   pmulticmds->Add(pcmd2);
   currentCore->Execute(pmulticmds);
-      
+
   /* tell the user we're done & provide short merge report */
-  const wxChar* cs_entries = (numUpdated == 1 ? _("entry") : _("entries"));
-  wxString resultStr = wxString::Format(_("\nSynchronize completed: %d %s updated"), numUpdated, cs_entries);
-  rpt.WriteLine(static_cast<const wxChar*>(resultStr));
+  const wxString cs_entries = (numUpdated == 1 ? _("entry") : _("entries"));
+  wxString resultStr = wxString::Format(_("\nSynchronize completed: %d %s updated").c_str(), numUpdated, cs_entries);
+  rpt.WriteLine(resultStr.c_str());
   rpt.EndReport();
 
   m_syncData->numUpdated = numUpdated;
@@ -751,20 +772,20 @@ void SyncStatusPage::ReportAdvancedOptions(CReport* rpt, const wxString& operati
   wxString line = m_syncData->selCriteria.GetGroupSelectionDescription();
   line << _(" were ") << operation << _(" with corresponding entries from \"")
               << m_syncData->otherDB.GetFullPath() << wxT('"');
-  rpt->WriteLine(static_cast<const wxChar*>(line));
-  
+  rpt->WriteLine(line.c_str());
+
   wxArrayString fieldsSelected, fieldsNotSelected;
   const bool allSelected = m_syncData->selCriteria.GetFieldSelection(fieldsSelected, fieldsNotSelected);
   if (allSelected) {
-    line.Printf(_("All fields in matching entries were %s"), static_cast<const wxChar*>(operation));
-    rpt->WriteLine(static_cast<const wxChar*>(line));
+    line.Printf(_("All fields in matching entries were %s").c_str(), operation);
+    rpt->WriteLine(line.c_str());
   }
   else {
-    line.Printf(_("The following fields were %s"), static_cast<const wxChar*>(operation));
-    rpt->WriteLine(static_cast<const wxChar*>(line));
+    line.Printf(_("The following fields were %s").c_str(), operation);
+    rpt->WriteLine(line.c_str());
     for( size_t idx = 0; idx < fieldsSelected.Count(); ++idx) {
-      line.Printf(_("\t* %s"), static_cast<const wxChar*>(fieldsSelected[idx]));
-      rpt->WriteLine(static_cast<const wxChar*>(line));
+      line.Printf(wxT("\t* %s"), fieldsSelected[idx]);
+      rpt->WriteLine(line.c_str());
     }
   }
 }
