@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+ * Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -36,10 +36,11 @@
 #include "pwsclip.h"
 #include "./wxutils.h"
 
+#include <algorithm>
+
 #ifdef __WXMSW__
 #include <wx/msw/msvcrt.h>
 #endif
-#include <algorithm>
 
 ////@begin XPM images
 ////@end XPM images
@@ -60,43 +61,26 @@ BEGIN_EVENT_TABLE( AddEditPropSheet, wxPropertySheetDialog )
   EVT_BUTTON( wxID_OK, AddEditPropSheet::OnOk )
 ////@begin AddEditPropSheet event table entries
   EVT_BUTTON( ID_BUTTON2, AddEditPropSheet::OnShowHideClick )
-
   EVT_BUTTON( ID_BUTTON3, AddEditPropSheet::OnGenerateButtonClick )
-
   EVT_BUTTON( ID_GO_BTN, AddEditPropSheet::OnGoButtonClick )
-
   EVT_BUTTON( ID_SEND_BTN, AddEditPropSheet::OnSendButtonClick )
-
   EVT_CHECKBOX( ID_CHECKBOX1, AddEditPropSheet::OnKeepHistoryClick )
-
   EVT_RADIOBUTTON( ID_RADIOBUTTON, AddEditPropSheet::OnRadiobuttonSelected )
-
   EVT_DATE_CHANGED( ID_DATECTRL, AddEditPropSheet::OnExpDateChanged )
-
   EVT_RADIOBUTTON( ID_RADIOBUTTON1, AddEditPropSheet::OnRadiobuttonSelected )
-
   EVT_SPINCTRL( ID_SPINCTRL2, AddEditPropSheet::OnExpIntervalChanged )
-
   EVT_RADIOBUTTON( ID_RADIOBUTTON4, AddEditPropSheet::OnRadiobuttonSelected )
-
   EVT_RADIOBUTTON( ID_RADIOBUTTON2, AddEditPropSheet::OnPWPRBSelected )
-
   EVT_COMBOBOX( ID_POLICYLIST, AddEditPropSheet::OnPolicylistSelected )
-
   EVT_RADIOBUTTON( ID_RADIOBUTTON3, AddEditPropSheet::OnPWPRBSelected )
-
+  EVT_CHECKBOX( ID_CHECKBOX3, AddEditPropSheet::OnLowercaseCB )
+  EVT_CHECKBOX( ID_CHECKBOX4, AddEditPropSheet::OnUppercaseCB )
+  EVT_CHECKBOX( ID_CHECKBOX5, AddEditPropSheet::OnDigitsCB )
   EVT_CHECKBOX( ID_CHECKBOX6, AddEditPropSheet::OnSymbolsCB )
-
-  EVT_RADIOBUTTON( IDC_USE_DEFAULTSYMBOLS, AddEditPropSheet::OnSymbolsRB )
-
-  EVT_RADIOBUTTON( IDC_USE_OWNSYMBOLS, AddEditPropSheet::OnSymbolsRB )
-
+  EVT_BUTTON( ID_RESET_SYMBOLS, AddEditPropSheet::OnResetSymbolsClick )
   EVT_CHECKBOX( ID_CHECKBOX7, AddEditPropSheet::OnEZreadCBClick )
-
   EVT_CHECKBOX( ID_CHECKBOX8, AddEditPropSheet::OnPronouceableCBClick )
-
   EVT_CHECKBOX( ID_CHECKBOX9, AddEditPropSheet::OnUseHexCBClick )
-
 ////@end AddEditPropSheet event table entries
   EVT_SPINCTRL(ID_SPINCTRL5, AddEditPropSheet::OnAtLeastChars)
   EVT_SPINCTRL(ID_SPINCTRL6, AddEditPropSheet::OnAtLeastChars)
@@ -227,7 +211,7 @@ class PolicyValidator : public MultiCheckboxValidator
 {
 public:
   PolicyValidator(int rbID, int ids[], size_t num,
-		  const wxString& msg, const wxString& title)
+      const wxString& msg, const wxString& title)
     : MultiCheckboxValidator(ids, num, msg, title), m_rbID(rbID) {}
   PolicyValidator(const PolicyValidator &other)
     : MultiCheckboxValidator(other), m_rbID(other.m_rbID) {}
@@ -239,7 +223,7 @@ public:
     if (win && win->IsEnabled()) {
       wxRadioButton* rb = wxDynamicCast(win, wxRadioButton);
       if (rb && rb->GetValue()) {
-	return true;
+  return true;
       }
     }
     return MultiCheckboxValidator::Validate(parent);
@@ -255,9 +239,6 @@ private:
 void AddEditPropSheet::CreateControls()
 {
 ////@begin AddEditPropSheet content construction
-  AddEditPropSheet* itemPropertySheetDialog1 = this;
-
-  UNREFERENCED_PARAMETER(itemPropertySheetDialog1); // Remove MS Compiler warning
 
   m_BasicPanel = new wxPanel( GetBookCtrl(), ID_PANEL_BASIC, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
   wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxVERTICAL);
@@ -273,9 +254,9 @@ void AddEditPropSheet::CreateControls()
 
   wxArrayString m_groupCtrlStrings;
   m_groupCtrl = new wxComboBox( m_BasicPanel, ID_COMBOBOX1, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_groupCtrlStrings, wxCB_DROPDOWN );
-  m_BasicFGSizer->Add(m_groupCtrl, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  m_BasicFGSizer->Add(m_groupCtrl, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  m_BasicFGSizer->Add(10, 10, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  m_BasicFGSizer->Add(10, 13, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   wxStaticText* itemStaticText9 = new wxStaticText( m_BasicPanel, wxID_STATIC, _("Title:"), wxDefaultPosition, wxDefaultSize, 0 );
   m_BasicFGSizer->Add(itemStaticText9, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
@@ -283,7 +264,7 @@ void AddEditPropSheet::CreateControls()
   wxTextCtrl* itemTextCtrl10 = new wxTextCtrl( m_BasicPanel, ID_TEXTCTRL5, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
   m_BasicFGSizer->Add(itemTextCtrl10, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  m_BasicFGSizer->Add(10, 10, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  m_BasicFGSizer->Add(10, 13, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   wxStaticText* itemStaticText12 = new wxStaticText( m_BasicPanel, wxID_STATIC, _("Username:"), wxDefaultPosition, wxDefaultSize, 0 );
   m_BasicFGSizer->Add(itemStaticText12, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
@@ -291,7 +272,7 @@ void AddEditPropSheet::CreateControls()
   wxTextCtrl* itemTextCtrl13 = new wxTextCtrl( m_BasicPanel, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
   m_BasicFGSizer->Add(itemTextCtrl13, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  m_BasicFGSizer->Add(10, 10, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  m_BasicFGSizer->Add(10, 13, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   wxStaticText* itemStaticText15 = new wxStaticText( m_BasicPanel, wxID_STATIC, _("Password:"), wxDefaultPosition, wxDefaultSize, 0 );
   m_BasicFGSizer->Add(itemStaticText15, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
@@ -301,12 +282,12 @@ void AddEditPropSheet::CreateControls()
 
   wxBoxSizer* itemBoxSizer17 = new wxBoxSizer(wxHORIZONTAL);
   m_BasicFGSizer->Add(itemBoxSizer17, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 0);
-  itemBoxSizer17->Add(10, 10, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  itemBoxSizer17->Add(10, 13, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   m_ShowHideCtrl = new wxButton( m_BasicPanel, ID_BUTTON2, _("&Hide"), wxDefaultPosition, wxDefaultSize, 0 );
   itemBoxSizer17->Add(m_ShowHideCtrl, 0, wxALIGN_CENTER_VERTICAL|wxALL, 0);
 
-  itemBoxSizer17->Add(10, 10, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  itemBoxSizer17->Add(10, 13, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   wxButton* itemButton21 = new wxButton( m_BasicPanel, ID_BUTTON3, _("&Generate"), wxDefaultPosition, wxDefaultSize, 0 );
   itemBoxSizer17->Add(itemButton21, 0, wxALIGN_CENTER_VERTICAL|wxALL, 0);
@@ -317,7 +298,7 @@ void AddEditPropSheet::CreateControls()
   m_Password2Ctrl = new wxTextCtrl( m_BasicPanel, ID_TEXTCTRL3, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_PASSWORD );
   m_BasicFGSizer->Add(m_Password2Ctrl, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  m_BasicFGSizer->Add(10, 10, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  m_BasicFGSizer->Add(10, 13, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   wxStaticText* itemStaticText25 = new wxStaticText( m_BasicPanel, wxID_STATIC, _("URL:"), wxDefaultPosition, wxDefaultSize, 0 );
   m_BasicFGSizer->Add(itemStaticText25, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
@@ -327,7 +308,7 @@ void AddEditPropSheet::CreateControls()
 
   wxBoxSizer* itemBoxSizer27 = new wxBoxSizer(wxHORIZONTAL);
   m_BasicFGSizer->Add(itemBoxSizer27, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 0);
-  itemBoxSizer27->Add(10, 10, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  itemBoxSizer27->Add(10, 13, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   wxButton* itemButton29 = new wxButton( m_BasicPanel, ID_GO_BTN, _("Go"), wxDefaultPosition, wxDefaultSize, 0 );
   itemBoxSizer27->Add(itemButton29, 0, wxALIGN_CENTER_VERTICAL|wxALL, 0);
@@ -340,15 +321,16 @@ void AddEditPropSheet::CreateControls()
 
   wxBoxSizer* itemBoxSizer32 = new wxBoxSizer(wxHORIZONTAL);
   m_BasicFGSizer->Add(itemBoxSizer32, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 0);
-  itemBoxSizer32->Add(10, 10, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  itemBoxSizer32->Add(10, 13, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   wxButton* itemButton34 = new wxButton( m_BasicPanel, ID_SEND_BTN, _("Send"), wxDefaultPosition, wxDefaultSize, 0 );
   itemBoxSizer32->Add(itemButton34, 0, wxALIGN_CENTER_VERTICAL|wxALL, 0);
 
   wxBoxSizer* itemBoxSizer35 = new wxBoxSizer(wxHORIZONTAL);
-  itemBoxSizer3->Add(itemBoxSizer35, 0, wxGROW|wxALL, 5);
+  itemBoxSizer3->Add(itemBoxSizer35, 1, wxGROW|wxALL, 5);
   wxStaticText* itemStaticText36 = new wxStaticText( m_BasicPanel, wxID_STATIC, _("Notes:"), wxDefaultPosition, wxDefaultSize, 0 );
-  itemBoxSizer35->Add(itemStaticText36, 1, wxALIGN_TOP|wxALL, 5);
+  itemBoxSizer35->Add(itemStaticText36, 0, wxALIGN_TOP|wxALL, 5);
+  itemBoxSizer35->Add(27, 13, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   m_noteTX = new wxTextCtrl( m_BasicPanel, ID_TEXTCTRL7, wxEmptyString, wxDefaultPosition, wxSize(-1, m_BasicPanel->ConvertDialogToPixels(wxSize(-1, 50)).y), wxTE_MULTILINE );
   itemBoxSizer35->Add(m_noteTX, 5, wxGROW|wxALL, 3);
@@ -433,7 +415,7 @@ void AddEditPropSheet::CreateControls()
   wxButton* itemButton56 = new wxButton( itemPanel38, ID_BUTTON1, _("Clear History"), wxDefaultPosition, wxDefaultSize, 0 );
   itemBoxSizer55->Add(itemButton56, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  itemBoxSizer55->Add(10, 10, 10, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  itemBoxSizer55->Add(10, 13, 10, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   wxButton* itemButton58 = new wxButton( itemPanel38, ID_BUTTON4, _("Copy All"), wxDefaultPosition, wxDefaultSize, 0 );
   itemBoxSizer55->Add(itemButton58, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
@@ -458,7 +440,7 @@ void AddEditPropSheet::CreateControls()
   m_ExpDate = new wxDatePickerCtrl( itemPanel59, ID_DATECTRL, wxDateTime(), wxDefaultPosition, wxDefaultSize, wxDP_DEFAULT );
   itemFlexGridSizer63->Add(m_ExpDate, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  itemFlexGridSizer63->Add(10, 10, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  itemFlexGridSizer63->Add(10, 13, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   m_InRB = new wxRadioButton( itemPanel59, ID_RADIOBUTTON1, _("In"), wxDefaultPosition, wxDefaultSize, 0 );
   m_InRB->SetValue(false);
@@ -532,7 +514,7 @@ void AddEditPropSheet::CreateControls()
   m_defPWPRB->SetValue(false);
   itemBoxSizer89->Add(m_defPWPRB, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  itemBoxSizer89->Add(10, 10, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  itemBoxSizer89->Add(10, 13, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   wxArrayString m_cbxPolicyNamesStrings;
   m_cbxPolicyNames = new wxComboBox( itemPanel87, ID_POLICYLIST, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_cbxPolicyNamesStrings, wxCB_READONLY );
@@ -553,7 +535,7 @@ void AddEditPropSheet::CreateControls()
   m_pwpLenCtrl = new wxSpinCtrl( itemPanel87, ID_SPINCTRL3, _T("12"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 4, 1024, 12 );
   itemBoxSizer95->Add(m_pwpLenCtrl, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  m_pwMinsGSzr = new wxGridSizer(6, 2, 0, 0);
+  m_pwMinsGSzr = new wxGridSizer(0, 2, 0, 0);
   itemStaticBoxSizer88->Add(m_pwMinsGSzr, 0, wxALIGN_LEFT|wxALL, 5);
   m_pwpUseLowerCtrl = new wxCheckBox( itemPanel87, ID_CHECKBOX3, _("Use lowercase letters"), wxDefaultPosition, wxDefaultSize, 0 );
   m_pwpUseLowerCtrl->SetValue(false);
@@ -564,7 +546,7 @@ void AddEditPropSheet::CreateControls()
   wxStaticText* itemStaticText101 = new wxStaticText( itemPanel87, wxID_STATIC, _("(At least "), wxDefaultPosition, wxDefaultSize, 0 );
   m_pwNumLCbox->Add(itemStaticText101, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  m_pwpLCSpin = new wxSpinCtrl( itemPanel87, ID_SPINCTRL5, _T("0"), wxDefaultPosition, wxSize(itemPanel87->ConvertDialogToPixels(wxSize(20, -1)).x, -1), wxSP_ARROW_KEYS, 0, 100, 0 );
+  m_pwpLCSpin = new wxSpinCtrl( itemPanel87, ID_SPINCTRL5, _T("0"), wxDefaultPosition, wxSize(itemPanel87->ConvertDialogToPixels(wxSize(40, -1)).x, -1), wxSP_ARROW_KEYS, 0, 100, 0 );
   m_pwNumLCbox->Add(m_pwpLCSpin, 0, wxALIGN_CENTER_VERTICAL|wxALL, 0);
 
   wxStaticText* itemStaticText103 = new wxStaticText( itemPanel87, wxID_STATIC, _(")"), wxDefaultPosition, wxDefaultSize, 0 );
@@ -579,7 +561,7 @@ void AddEditPropSheet::CreateControls()
   wxStaticText* itemStaticText106 = new wxStaticText( itemPanel87, wxID_STATIC, _("(At least "), wxDefaultPosition, wxDefaultSize, 0 );
   m_pwNumUCbox->Add(itemStaticText106, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  m_pwpUCSpin = new wxSpinCtrl( itemPanel87, ID_SPINCTRL6, _T("0"), wxDefaultPosition, wxSize(itemPanel87->ConvertDialogToPixels(wxSize(20, -1)).x, -1), wxSP_ARROW_KEYS, 0, 100, 0 );
+  m_pwpUCSpin = new wxSpinCtrl( itemPanel87, ID_SPINCTRL6, _T("0"), wxDefaultPosition, wxSize(itemPanel87->ConvertDialogToPixels(wxSize(40, -1)).x, -1), wxSP_ARROW_KEYS, 0, 100, 0 );
   m_pwNumUCbox->Add(m_pwpUCSpin, 0, wxALIGN_CENTER_VERTICAL|wxALL, 0);
 
   wxStaticText* itemStaticText108 = new wxStaticText( itemPanel87, wxID_STATIC, _(")"), wxDefaultPosition, wxDefaultSize, 0 );
@@ -594,7 +576,7 @@ void AddEditPropSheet::CreateControls()
   wxStaticText* itemStaticText111 = new wxStaticText( itemPanel87, wxID_STATIC, _("(At least "), wxDefaultPosition, wxDefaultSize, 0 );
   m_pwNumDigbox->Add(itemStaticText111, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  m_pwpDigSpin = new wxSpinCtrl( itemPanel87, ID_SPINCTRL7, _T("0"), wxDefaultPosition, wxSize(itemPanel87->ConvertDialogToPixels(wxSize(20, -1)).x, -1), wxSP_ARROW_KEYS, 0, 100, 0 );
+  m_pwpDigSpin = new wxSpinCtrl( itemPanel87, ID_SPINCTRL7, _T("0"), wxDefaultPosition, wxSize(itemPanel87->ConvertDialogToPixels(wxSize(40, -1)).x, -1), wxSP_ARROW_KEYS, 0, 100, 0 );
   m_pwNumDigbox->Add(m_pwpDigSpin, 0, wxALIGN_CENTER_VERTICAL|wxALL, 0);
 
   wxStaticText* itemStaticText113 = new wxStaticText( itemPanel87, wxID_STATIC, _(")"), wxDefaultPosition, wxDefaultSize, 0 );
@@ -609,40 +591,32 @@ void AddEditPropSheet::CreateControls()
   wxStaticText* itemStaticText116 = new wxStaticText( itemPanel87, wxID_STATIC, _("(At least "), wxDefaultPosition, wxDefaultSize, 0 );
   m_pwNumSymbox->Add(itemStaticText116, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  m_pwpSymSpin = new wxSpinCtrl( itemPanel87, ID_SPINCTRL8, _T("0"), wxDefaultPosition, wxSize(itemPanel87->ConvertDialogToPixels(wxSize(20, -1)).x, -1), wxSP_ARROW_KEYS, 0, 100, 0 );
+  m_pwpSymSpin = new wxSpinCtrl( itemPanel87, ID_SPINCTRL8, _T("0"), wxDefaultPosition, wxSize(itemPanel87->ConvertDialogToPixels(wxSize(40, -1)).x, -1), wxSP_ARROW_KEYS, 0, 100, 0 );
   m_pwNumSymbox->Add(m_pwpSymSpin, 0, wxALIGN_CENTER_VERTICAL|wxALL, 0);
 
   wxStaticText* itemStaticText118 = new wxStaticText( itemPanel87, wxID_STATIC, _(")"), wxDefaultPosition, wxDefaultSize, 0 );
   m_pwNumSymbox->Add(itemStaticText118, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  wxRadioButton* itemRadioButton119 = new wxRadioButton( itemPanel87, IDC_USE_DEFAULTSYMBOLS, _("Default set"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP );
-  itemRadioButton119->SetValue(false);
-  m_pwMinsGSzr->Add(itemRadioButton119, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
-
-  wxStaticText* itemStaticText120 = new wxStaticText( itemPanel87, IDC_STATIC_DEFAULT_SYMBOLS, _("Static text"), wxDefaultPosition, wxSize(itemPanel87->ConvertDialogToPixels(wxSize(120, -1)).x, -1), 0 );
-  m_pwMinsGSzr->Add(itemStaticText120, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 0);
-
-  wxRadioButton* itemRadioButton121 = new wxRadioButton( itemPanel87, IDC_USE_OWNSYMBOLS, _("Special set"), wxDefaultPosition, wxDefaultSize, 0 );
-  itemRadioButton121->SetValue(false);
-  m_pwMinsGSzr->Add(itemRadioButton121, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
-
-  m_ownsymbols = new wxTextCtrl( itemPanel87, IDC_OWNSYMBOLS, wxEmptyString, wxDefaultPosition, wxSize(itemPanel87->ConvertDialogToPixels(wxSize(60, -1)).x, -1), 0 );
+  m_ownsymbols = new wxTextCtrl( itemPanel87, IDC_OWNSYMBOLS, wxEmptyString, wxDefaultPosition, wxSize(itemPanel87->ConvertDialogToPixels(wxSize(120, -1)).x, -1), 0 );
   m_pwMinsGSzr->Add(m_ownsymbols, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 0);
 
+  wxButton* itemButton120 = new wxButton( itemPanel87, ID_RESET_SYMBOLS, _("Reset"), wxDefaultPosition, wxDefaultSize, 0 );
+  m_pwMinsGSzr->Add(itemButton120, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 0);
+
   m_pwpEasyCtrl = new wxCheckBox( itemPanel87, ID_CHECKBOX7, _("Use only easy-to-read characters\n(i.e., no 'l', '1', etc.)"), wxDefaultPosition, wxDefaultSize, 0 );
   m_pwpEasyCtrl->SetValue(false);
   m_pwMinsGSzr->Add(m_pwpEasyCtrl, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 0);
 
-  m_pwMinsGSzr->Add(10, 10, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 0);
+  m_pwMinsGSzr->Add(10, 13, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 0);
 
   m_pwpPronounceCtrl = new wxCheckBox( itemPanel87, ID_CHECKBOX8, _("Generate pronounceable passwords"), wxDefaultPosition, wxDefaultSize, 0 );
   m_pwpPronounceCtrl->SetValue(false);
   m_pwMinsGSzr->Add(m_pwpPronounceCtrl, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 0);
 
-  m_pwMinsGSzr->Add(10, 10, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 0);
+  m_pwMinsGSzr->Add(10, 13, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 0);
 
-  wxStaticText* itemStaticText127 = new wxStaticText( itemPanel87, wxID_STATIC, _("Or"), wxDefaultPosition, wxDefaultSize, 0 );
-  itemStaticBoxSizer88->Add(itemStaticText127, 0, wxALIGN_LEFT|wxALL, 5);
+  wxStaticText* itemStaticText125 = new wxStaticText( itemPanel87, wxID_STATIC, _("Or"), wxDefaultPosition, wxDefaultSize, 0 );
+  itemStaticBoxSizer88->Add(itemStaticText125, 0, wxALIGN_LEFT|wxALL, 5);
 
   m_pwpHexCtrl = new wxCheckBox( itemPanel87, ID_CHECKBOX9, _("Use hexadecimal digits only (0-9, a-f)"), wxDefaultPosition, wxDefaultSize, 0 );
   m_pwpHexCtrl->SetValue(false);
@@ -667,8 +641,6 @@ void AddEditPropSheet::CreateControls()
   itemStaticText82->SetValidator( wxGenericValidator(& m_PMTime) );
   itemStaticText84->SetValidator( wxGenericValidator(& m_ATime) );
   itemStaticText86->SetValidator( wxGenericValidator(& m_RMTime) );
-  itemRadioButton119->SetValidator( wxGenericValidator(& m_usedefaultsymbols) );
-  itemRadioButton121->SetValidator( wxGenericValidator(& m_useownsymbols) );
   m_ownsymbols->SetValidator( wxGenericValidator(& m_symbols) );
   // Connect events and objects
   m_noteTX->Connect(ID_TEXTCTRL7, wxEVT_SET_FOCUS, wxFocusEventHandler(AddEditPropSheet::OnNoteSetFocus), NULL, this);
@@ -680,14 +652,8 @@ void AddEditPropSheet::CreateControls()
   m_PWHgrid->SetColLabelValue(1, _("Password"));
 
   // Setup symbols
-  StringX sx_symbols = PWSprefs::GetInstance()->GetPref(PWSprefs::DefaultSymbols);
-  if (sx_symbols.empty()) {
-    stringT st_symbols;
-    CPasswordCharPool::GetDefaultSymbols(st_symbols);
-    sx_symbols = st_symbols.c_str();
-  }
-
-  FindWindow(IDC_STATIC_DEFAULT_SYMBOLS)->SetLabel(sx_symbols.c_str());
+  m_symbols = CPasswordCharPool::GetDefaultSymbols().c_str();
+  m_ownsymbols->SetValue(m_symbols);
 }
 
 /*!
@@ -703,11 +669,10 @@ bool AddEditPropSheet::ShowToolTips()
  * Get bitmap resources
  */
 
-wxBitmap AddEditPropSheet::GetBitmapResource( const wxString& name )
+wxBitmap AddEditPropSheet::GetBitmapResource( const wxString& WXUNUSED(name) )
 {
   // Bitmap retrieval
 ////@begin AddEditPropSheet bitmap retrieval
-  wxUnusedVar(name);
   return wxNullBitmap;
 ////@end AddEditPropSheet bitmap retrieval
 }
@@ -716,11 +681,10 @@ wxBitmap AddEditPropSheet::GetBitmapResource( const wxString& name )
  * Get icon resources
  */
 
-wxIcon AddEditPropSheet::GetIconResource( const wxString& name )
+wxIcon AddEditPropSheet::GetIconResource( const wxString& WXUNUSED(name) )
 {
   // Icon retrieval
 ////@begin AddEditPropSheet icon retrieval
-  wxUnusedVar(name);
   return wxNullIcon;
 ////@end AddEditPropSheet icon retrieval
 }
@@ -760,10 +724,6 @@ void AddEditPropSheet::UpdatePWPolicyControls(const PWPolicy& pwp)
   bUseVal = (pwp.flags & PWPolicy::UseSymbols) != 0;
   m_pwpSymCtrl->SetValue(bUseVal);
   m_pwpSymSpin->SetValue(pwp.symbolminlength);
-  FindWindow(IDC_USE_DEFAULTSYMBOLS)->Enable(bUseVal);
-  FindWindow(IDC_STATIC_DEFAULT_SYMBOLS)->Enable(bUseVal);
-  FindWindow(IDC_USE_OWNSYMBOLS)->Enable(bUseVal);
-  FindWindow(IDC_OWNSYMBOLS)->Enable(bUseVal);
 
   bUseVal = (pwp.flags & PWPolicy::UseEasyVision) != 0;
   m_pwpEasyCtrl->SetValue(bUseVal);
@@ -774,12 +734,7 @@ void AddEditPropSheet::UpdatePWPolicyControls(const PWPolicy& pwp)
 
   ShowPWPSpinners(!m_pwpPronounceCtrl->GetValue() && !m_pwpEasyCtrl->GetValue());
 
-  if (pwp.symbols.empty()) {
-    static_cast<wxRadioButton*>(FindWindow(IDC_USE_DEFAULTSYMBOLS))->SetValue(true);
-    static_cast<wxRadioButton*>(FindWindow(IDC_USE_OWNSYMBOLS))->SetValue(false);
-  } else {
-    static_cast<wxRadioButton*>(FindWindow(IDC_USE_DEFAULTSYMBOLS))->SetValue(false);
-    static_cast<wxRadioButton*>(FindWindow(IDC_USE_OWNSYMBOLS))->SetValue(true);
+  if (!pwp.symbols.empty()) {
     m_symbols = pwp.symbols.c_str();
     Validate(); TransferDataToWindow();
   }
@@ -790,6 +745,16 @@ void AddEditPropSheet::EnablePWPolicyControls(bool enable)
   m_pwpLenCtrl->Enable(enable);
   EnableSizerChildren(m_pwMinsGSzr, enable && !m_pwpHexCtrl->GetValue());
   m_pwpHexCtrl->Enable(enable);
+  if (enable) {
+    // Be more specific for character set controls
+    m_pwpLCSpin->Enable(m_pwpUseLowerCtrl->GetValue());
+    m_pwpUCSpin->Enable(m_pwpUseUpperCtrl->GetValue());
+    m_pwpDigSpin->Enable(m_pwpUseDigitsCtrl->GetValue());
+    bool useSyms = m_pwpSymCtrl->GetValue();
+    m_ownsymbols->Enable(useSyms);
+    m_pwpSymSpin->Enable(useSyms);
+    FindWindow(ID_RESET_SYMBOLS)->Enable(useSyms);
+  }
 }
 
 struct newer {
@@ -821,8 +786,8 @@ static struct {short pv; wxString name;}
 
   bool useDefault = (dca < PWSprefs::minDCA || dca > PWSprefs::maxDCA);
   short defDCA =  short(PWSprefs::GetInstance()->
-			GetPref(isShift ?
-				PWSprefs::ShiftDoubleClickAction : PWSprefs::DoubleClickAction));
+      GetPref(isShift ?
+        PWSprefs::ShiftDoubleClickAction : PWSprefs::DoubleClickAction));
   if (useDefault) {
     iDCA = defDCA;
   } else {
@@ -836,7 +801,7 @@ static struct {short pv; wxString name;}
     pcbox->SetClientData(i, reinterpret_cast<void *>(dcaMapping[i].pv));
     if (dcaMapping[i].pv == defDCA) {
       wxString dv = dcaMapping[i].name;
-      dv += _(" ("); dv += _("default"); dv += _(")");
+      dv += wxT(" ("); dv += _("default"); dv += wxT(")");
       pcbox->SetString(i, dv);
     }
     if (iDCA == dcaMapping[i].pv)
@@ -1012,10 +977,8 @@ void AddEditPropSheet::ItemFieldsToPropSheet()
   if (specificPwPolicy) {
     m_item.GetPWPolicy(policy);
     policy.symbols = m_item.GetSymbols().c_str();
-    m_usedefaultsymbols = policy.symbols.empty();
-    m_useownsymbols = !m_usedefaultsymbols;
-    if (m_useownsymbols)
-      m_ownsymbols->SetValue(policy.symbols.c_str());
+    if (!policy.symbols.empty())
+      m_symbols = policy.symbols.c_str();
   } else { // no item-specific policy, either default or named
     // Select item's named policy, or Default
     const wxString itemPolName = m_item.GetPolicyName().c_str();
@@ -1073,7 +1036,7 @@ void AddEditPropSheet::OnGenerateButtonClick( wxCommandEvent& /* evt */ )
 
 void AddEditPropSheet::OnShowHideClick( wxCommandEvent& /* evt */ )
 {
-  m_password = m_PasswordCtrl->GetValue(); // save visible password
+  m_password = m_PasswordCtrl->GetValue().c_str(); // save visible password
   if (m_isPWHidden) {
     ShowPassword();
   } else {
@@ -1159,8 +1122,9 @@ void AddEditPropSheet::OnOk(wxCommandEvent& /* evt */)
       else
         m_PasswordCtrl->SetFocus();
 
-      wxMessageBox(wxString(_("This entry must have a ")) << (m_title.IsEmpty() ? _("title"): _("password")),
-                    _("Error"), wxOK|wxICON_INFORMATION, this);
+      wxMessageBox(wxString::Format(wxString(_("This entry must have a %s")),
+                                    (m_title.IsEmpty() ? _("title"): _("password"))),
+                   _("Error"), wxOK|wxICON_INFORMATION, this);
       return;
     }
 
@@ -1212,7 +1176,7 @@ void AddEditPropSheet::OnOk(wxCommandEvent& /* evt */)
                                 pwhl, PWSUtil::TMC_LOCALE);
 
       // Create a new PWHistory header, as per settings in this dialog
-      size_t numEntries = MIN(pwhl.size(), static_cast<size_t>(m_maxPWHist));
+      size_t numEntries = std::min(pwhl.size(), static_cast<size_t>(m_maxPWHist));
       m_PWHistory = towxstring(MakePWHistoryHeader(m_keepPWHist, m_maxPWHist, numEntries));
       //reverse-sort the history entries to retain only the newest
       std::sort(pwhl.begin(), pwhl.end(), newer());
@@ -1578,8 +1542,7 @@ PWPolicy AddEditPropSheet::GetPWPolicyFromUI()
   if (m_pwpHexCtrl->GetValue())
     pwp.flags = PWPolicy::UseHexDigits; //yes, its '=' and not '|='
 
-  if (m_useownsymbols)
-    pwp.symbols = m_symbols.c_str();
+  pwp.symbols = m_symbols.c_str();
 
   return pwp;
 }
@@ -1656,14 +1619,15 @@ void AddEditPropSheet::OnEZreadCBClick(wxCommandEvent& evt)
       return;
     }
 
-    CPasswordCharPool::GetEasyVisionSymbols(st_symbols);
+    st_symbols = CPasswordCharPool::GetEasyVisionSymbols();
   } else { // not checked - restore default symbols to appropriate value
     if (m_pwpPronounceCtrl->GetValue())
-      CPasswordCharPool::GetPronounceableSymbols(st_symbols);
+      st_symbols = CPasswordCharPool::GetPronounceableSymbols();
     else
-      CPasswordCharPool::GetDefaultSymbols(st_symbols);
+      st_symbols = CPasswordCharPool::GetDefaultSymbols();
   }
-  FindWindow(IDC_STATIC_DEFAULT_SYMBOLS)->SetLabel(st_symbols.c_str());
+  m_symbols = st_symbols.c_str();
+  m_ownsymbols->SetValue(m_symbols);
 }
 
 
@@ -1682,45 +1646,17 @@ void AddEditPropSheet::OnPronouceableCBClick( wxCommandEvent& evt)
                    _("Error"), wxOK|wxICON_ERROR, this);
       return;
     }
-    CPasswordCharPool::GetPronounceableSymbols(st_symbols);
+    st_symbols = CPasswordCharPool::GetPronounceableSymbols();
   } else { // not checked - restore default symbols to appropriate value
     if (m_pwpEasyCtrl->GetValue())
-      CPasswordCharPool::GetEasyVisionSymbols(st_symbols);
+      st_symbols = CPasswordCharPool::GetEasyVisionSymbols();
     else
-      CPasswordCharPool::GetDefaultSymbols(st_symbols);
+      st_symbols = CPasswordCharPool::GetDefaultSymbols();
   }
-  FindWindow(IDC_STATIC_DEFAULT_SYMBOLS)->SetLabel(st_symbols.c_str());
+  m_symbols = st_symbols.c_str();
+  m_ownsymbols->SetValue(m_symbols);
 }
 
-
-/*!
- * wxEVT_COMMAND_RADIOBUTTON_SELECTED event handler for ID_RADIOBUTTON8
- */
-
-void AddEditPropSheet::OnSymbolsRB( wxCommandEvent& evt)
-{
-  m_useownsymbols = (evt.GetId() == IDC_USE_OWNSYMBOLS);
-
-  if (m_useownsymbols)
-    FindWindow(IDC_OWNSYMBOLS)->SetFocus();
-}
-
-
-
-
-/*!
- * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX6
- */
-
-void AddEditPropSheet::OnSymbolsCB( wxCommandEvent& evt )
-{
-  bool enable = evt.IsChecked();
-  FindWindow(IDC_USE_DEFAULTSYMBOLS)->Enable(enable);
-  FindWindow(IDC_STATIC_DEFAULT_SYMBOLS)->Enable(enable);
-  FindWindow(IDC_USE_OWNSYMBOLS)->Enable(enable);
-}
-
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON9
  */
@@ -1778,7 +1714,7 @@ void AddEditPropSheet::OnPolicylistSelected( wxCommandEvent& event )
     policy = PWSprefs::GetInstance()->GetDefaultPolicy();
   } else {
     if (!m_core.GetPolicyFromName(tostringx(polName), policy)) {
-      pws_os::Trace(_("Couldn't find policy %s\n"), ToStr(polName));
+      pws_os::Trace(wxT("Couldn't find policy %s\n"), ToStr(polName));
       return;
     }
   }
@@ -1788,16 +1724,6 @@ void AddEditPropSheet::OnPolicylistSelected( wxCommandEvent& event )
 }
 
 
-/*!
- * wxEVT_SET_FOCUS event handler for IDC_OWNSYMBOLS
- */
-
-void AddEditPropSheet::OnOwnSymSetFocus( wxFocusEvent& )
-{
-  static_cast<wxRadioButton*>(FindWindow(IDC_USE_OWNSYMBOLS))->SetValue(true);
-}
-
-
 /*!
  * wxEVT_DATE_CHANGED event handler for ID_DATECTRL
  */
@@ -1817,3 +1743,77 @@ void AddEditPropSheet::OnExpIntervalChanged( wxSpinEvent& event )
   SetXTime(event.GetEventObject());
 }
 
+
+/*!
+ * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX6
+ */
+
+void AddEditPropSheet::OnSymbolsCB( wxCommandEvent& event )
+{
+  bool checked = event.IsChecked();
+  m_ownsymbols->Enable(checked);
+  m_pwpSymSpin->Enable(checked);
+  FindWindow(ID_RESET_SYMBOLS)->Enable(checked);
+}
+
+
+/*!
+ * wxEVT_SET_FOCUS event handler for IDC_OWNSYMBOLS
+ */
+
+void AddEditPropSheet::OnOwnSymSetFocus( wxFocusEvent& event )
+{
+////@begin wxEVT_SET_FOCUS event handler for IDC_OWNSYMBOLS in AddEditPropSheet.
+  // Before editing this code, remove the block markers.
+  event.Skip();
+////@end wxEVT_SET_FOCUS event handler for IDC_OWNSYMBOLS in AddEditPropSheet.
+}
+
+
+/*!
+ * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_RESET_SYMBOLS
+ */
+
+void AddEditPropSheet::OnResetSymbolsClick( wxCommandEvent& WXUNUSED(event) )
+{
+  stringT st_symbols;
+  if (m_pwpEasyCtrl->GetValue())
+    st_symbols = CPasswordCharPool::GetEasyVisionSymbols();
+  else if (m_pwpPronounceCtrl->GetValue())
+    st_symbols = CPasswordCharPool::GetPronounceableSymbols();
+  else
+    st_symbols = CPasswordCharPool::GetDefaultSymbols();
+  m_symbols = st_symbols.c_str();
+  m_ownsymbols->SetValue(m_symbols);
+}
+
+
+/*!
+ * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX5
+ */
+
+void AddEditPropSheet::OnDigitsCB( wxCommandEvent& event )
+{
+  m_pwpDigSpin->Enable(event.IsChecked());
+}
+
+
+/*!
+ * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX4
+ */
+
+void AddEditPropSheet::OnUppercaseCB( wxCommandEvent& event )
+{
+  m_pwpUCSpin->Enable(event.IsChecked());
+}
+
+
+/*!
+ * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX3
+ */
+
+void AddEditPropSheet::OnLowercaseCB( wxCommandEvent& event )
+{
+  m_pwpLCSpin->Enable(event.IsChecked());
+}
+
