@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
+ * Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -31,15 +31,17 @@
 
 ////@begin forward declarations
 class wxSpinCtrl;
-class wxGridSizer;
-class wxBoxSizer;
-class wxBookCtrlEvent;
 ////@end forward declarations
+class wxBookCtrlEvent;
 
 /*!
  * Control identifiers
  */
 
+#ifndef wxDIALOG_MODAL
+#define wxDIALOG_MODAL 0
+#endif
+
 ////@begin control identifiers
 #define ID_OPTIONS 10130
 #define ID_PANEL 10131
@@ -71,6 +73,7 @@ class wxBookCtrlEvent;
 #define ID_CHECKBOX21 10161
 #define ID_CHECKBOX22 10162
 #define ID_COMBOBOX3 10163
+#define ID_COMBOBOX 10006
 #define ID_CHECKBOX23 10164
 #define ID_TEXTCTRL11 10165
 #define ID_CHECKBOX24 10166
@@ -79,19 +82,6 @@ class wxBookCtrlEvent;
 #define ID_TEXTCTRL13 10169
 #define ID_BUTTON8 10170
 #define ID_TEXTCTRL14 10171
-#define ID_PANEL3 10134
-#define ID_SPINCTRL3 10117
-#define ID_CHECKBOX3 10118
-#define ID_SPINCTRL5 10126
-#define ID_CHECKBOX4 10119
-#define ID_SPINCTRL6 10127
-#define ID_CHECKBOX5 10120
-#define ID_SPINCTRL7 10128
-#define ID_CHECKBOX6 10121
-#define ID_SPINCTRL8 10129
-#define ID_CHECKBOX7 10122
-#define ID_CHECKBOX8 10123
-#define ID_CHECKBOX9 10124
 #define ID_PANEL4 10135
 #define ID_CHECKBOX26 10172
 #define ID_SPINCTRL11 10173
@@ -107,6 +97,7 @@ class wxBookCtrlEvent;
 #define ID_CHECKBOX28 10003
 #define ID_CHECKBOX29 10180
 #define ID_SPINCTRL12 10181
+#define ID_SLIDER 10059
 #define ID_PANEL6 10137
 #define ID_CHECKBOX30 10182
 #define ID_SPINCTRL13 10183
@@ -115,16 +106,11 @@ class wxBookCtrlEvent;
 #define ID_CHECKBOX32 10185
 #define ID_CHECKBOX33 10186
 #define ID_CHECKBOX34 10005
-#if defined(__X__) || defined(__WXGTK__)
-#define ID_CHECKBOX35 10006
-#endif
+#define ID_CHECKBOX39 10010
+#define ID_CHECKBOX40 10114
 #define ID_PANEL7 10138
 #define ID_GRID1 10187
-#if WXWIN_COMPATIBILITY_2_6
 #define SYMBOL_COPTIONS_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxDIALOG_MODAL
-#else
-#define SYMBOL_COPTIONS_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX
-#endif
 #define SYMBOL_COPTIONS_TITLE _("Options")
 #define SYMBOL_COPTIONS_IDNAME ID_OPTIONS
 #define SYMBOL_COPTIONS_SIZE wxSize(400, 300)
@@ -161,115 +147,90 @@ public:
 ////@begin COptions event handler declarations
 
   /// wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX11
-  void OnBackupB4SaveClick( wxCommandEvent& evt );
+  void OnBackupB4SaveClick( wxCommandEvent& event );
 
   /// wxEVT_COMMAND_RADIOBUTTON_SELECTED event handler for ID_RADIOBUTTON4
-  void OnBuPrefix( wxCommandEvent& evt );
+  void OnBuPrefix( wxCommandEvent& event );
 
   /// wxEVT_SET_FOCUS event handler for ID_TEXTCTRL9
-  void OnBuPrefixTxtSetFocus( wxFocusEvent& evt );
+  void OnBuPrefixTxtSetFocus( wxFocusEvent& event );
 
   /// wxEVT_COMMAND_COMBOBOX_SELECTED event handler for ID_COMBOBOX2
-  void OnSuffixCBSet( wxCommandEvent& evt );
+  void OnSuffixCBSet( wxCommandEvent& event );
 
   /// wxEVT_COMMAND_RADIOBUTTON_SELECTED event handler for ID_RADIOBUTTON6
-  void OnBuDirRB( wxCommandEvent& evt );
+  void OnBuDirRB( wxCommandEvent& event );
 
   /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON
-  void OnBuDirBrowseClick( wxCommandEvent& evt );
+  void OnBuDirBrowseClick( wxCommandEvent& event );
 
   /// wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX13
-  void OnShowUsernameInTreeCB( wxCommandEvent& evt );
+  void OnShowUsernameInTreeCB( wxCommandEvent& event );
 
   /// wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX19
-  void OnPreExpiryWarnClick( wxCommandEvent& evt );
+  void OnPreExpiryWarnClick( wxCommandEvent& event );
 
   /// wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX24
-  void OnUseDefaultUserClick( wxCommandEvent& evt );
+  void OnUseDefaultUserClick( wxCommandEvent& event );
 
   /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON8
-  void OnBrowseLocationClick( wxCommandEvent& evt );
-
-  /// wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX3
-  void OnPwPolUseClick( wxCommandEvent& evt );
+  void OnBrowseLocationClick( wxCommandEvent& event );
 
   /// wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX26
-  void OnPWHistSaveClick( wxCommandEvent& evt );
+  void OnPWHistSaveClick( wxCommandEvent& event );
 
   /// wxEVT_COMMAND_RADIOBUTTON_SELECTED event handler for ID_PWHISTNOCHANGE
-  void OnPWHistRB( wxCommandEvent& evt );
+  void OnPWHistRB( wxCommandEvent& event );
 
   /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_PWHISTNOCHANGE
-  void OnPWHistApply( wxCommandEvent& evt );
+  void OnPWHistApply( wxCommandEvent& event );
 
   /// wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX29
-  void OnLockOnIdleClick( wxCommandEvent& evt );
+  void OnLockOnIdleClick( wxCommandEvent& event );
 
   /// wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX30
-  void OnUseSystrayClick( wxCommandEvent& evt );
+  void OnUseSystrayClick( wxCommandEvent& event );
+
+////@end COptions event handler declarations
 
   /// wxEVT_COMMAND_BOOKCTRL_PAGE_CHANGING event handler for all pages (wxID_ANY)
   void OnPageChanging(wxBookCtrlEvent& evt);
 
-  void OnAtLeastChars(wxSpinEvent& evt);
-
-////@end COptions event handler declarations
 
 ////@begin COptions member function declarations
 
-  bool GetSaveimmediate() const { return m_saveimmediate ; }
-  void SetSaveimmediate(bool value) { m_saveimmediate = value ; }
-
-  bool GetBackupb4save() const { return m_backupb4save ; }
-  void SetBackupb4save(bool value) { m_backupb4save = value ; }
-
   bool GetAlwaysontop() const { return m_alwaysontop ; }
   void SetAlwaysontop(bool value) { m_alwaysontop = value ; }
 
-  bool GetShowusernameintree() const { return m_showusernameintree ; }
-  void SetShowusernameintree(bool value) { m_showusernameintree = value ; }
+  wxString GetAutotypeStr() const { return m_autotypeStr ; }
+  void SetAutotypeStr(wxString value) { m_autotypeStr = value ; }
 
-  bool GetShownotesastipsinviews() const { return m_shownotesastipsinviews ; }
-  void SetShownotesastipsinviews(bool value) { m_shownotesastipsinviews = value ; }
-
-  bool GetPwshowinedit() const { return m_pwshowinedit ; }
-  void SetPwshowinedit(bool value) { m_pwshowinedit = value ; }
-
-  bool GetWordwrapnotes() const { return m_wordwrapnotes ; }
-  void SetWordwrapnotes(bool value) { m_wordwrapnotes = value ; }
-
-  int GetInittreeview() const { return m_inittreeview ; }
-  void SetInittreeview(int value) { m_inittreeview = value ; }
-
-  bool GetPreexpirywarn() const { return m_preexpirywarn ; }
-  void SetPreexpirywarn(bool value) { m_preexpirywarn = value ; }
-
-  bool GetNotesshowinedit() const { return m_notesshowinedit ; }
-  void SetNotesshowinedit(bool value) { m_notesshowinedit = value ; }
+  bool GetBackupb4save() const { return m_backupb4save ; }
+  void SetBackupb4save(bool value) { m_backupb4save = value ; }
 
   bool GetConfirmdelete() const { return m_confirmdelete ; }
   void SetConfirmdelete(bool value) { m_confirmdelete = value ; }
 
-  bool GetMaintaindatetimestamps() const { return m_maintaindatetimestamps ; }
-  void SetMaintaindatetimestamps(bool value) { m_maintaindatetimestamps = value ; }
+  int GetDoubleclickaction() const { return m_doubleclickaction ; }
+  void SetDoubleclickaction(int value) { m_doubleclickaction = value ; }
 
   bool GetEscexits() const { return m_escexits ; }
   void SetEscexits(bool value) { m_escexits = value ; }
 
-  int GetDoubleclickaction() const { return m_doubleclickaction ; }
-  void SetDoubleclickaction(int value) { m_doubleclickaction = value ; }
+  int GetHashIterSlider() const { return m_hashIterSlider ; }
+  void SetHashIterSlider(int value) { m_hashIterSlider = value ; }
+
+  int GetInittreeview() const { return m_inittreeview ; }
+  void SetInittreeview(int value) { m_inittreeview = value ; }
+
+  bool GetMaintaindatetimestamps() const { return m_maintaindatetimestamps ; }
+  void SetMaintaindatetimestamps(bool value) { m_maintaindatetimestamps = value ; }
 
   bool GetMinauto() const { return m_minauto ; }
   void SetMinauto(bool value) { m_minauto = value ; }
 
-  wxString GetAutotypeStr() const { return m_autotypeStr ; }
-  void SetAutotypeStr(wxString value) { m_autotypeStr = value ; }
-
-  bool GetUsedefuser() const { return m_usedefuser ; }
-  void SetUsedefuser(bool value) { m_usedefuser = value ; }
-
-  bool GetQuerysetdef() const { return m_querysetdef ; }
-  void SetQuerysetdef(bool value) { m_querysetdef = value ; }
+  bool GetNotesshowinedit() const { return m_notesshowinedit ; }
+  void SetNotesshowinedit(bool value) { m_notesshowinedit = value ; }
 
   wxString GetOtherbrowser() const { return m_otherbrowser ; }
   void SetOtherbrowser(wxString value) { m_otherbrowser = value ; }
@@ -277,26 +238,50 @@ public:
   wxString GetOtherbrowserparams() const { return m_otherbrowserparams ; }
   void SetOtherbrowserparams(wxString value) { m_otherbrowserparams = value ; }
 
+  bool GetPreexpirywarn() const { return m_preexpirywarn ; }
+  void SetPreexpirywarn(bool value) { m_preexpirywarn = value ; }
+
+  bool GetPutgroups1st() const { return m_putgroups1st ; }
+  void SetPutgroups1st(bool value) { m_putgroups1st = value ; }
+
   int GetPwdefaultlength() const { return m_pwdefaultlength ; }
   void SetPwdefaultlength(int value) { m_pwdefaultlength = value ; }
 
-  bool GetSecclrclponmin() const { return m_secclrclponmin ; }
-  void SetSecclrclponmin(bool value) { m_secclrclponmin = value ; }
+  bool GetPwshowinedit() const { return m_pwshowinedit ; }
+  void SetPwshowinedit(bool value) { m_pwshowinedit = value ; }
+
+  bool GetQuerysetdef() const { return m_querysetdef ; }
+  void SetQuerysetdef(bool value) { m_querysetdef = value ; }
+
+  bool GetSaveimmediate() const { return m_saveimmediate ; }
+  void SetSaveimmediate(bool value) { m_saveimmediate = value ; }
 
   bool GetSecclrclponexit() const { return m_secclrclponexit ; }
   void SetSecclrclponexit(bool value) { m_secclrclponexit = value ; }
 
-  bool GetSeclockonmin() const { return m_seclockonmin ; }
-  void SetSeclockonmin(bool value) { m_seclockonmin = value ; }
+  bool GetSecclrclponmin() const { return m_secclrclponmin ; }
+  void SetSecclrclponmin(bool value) { m_secclrclponmin = value ; }
 
   bool GetSecconfrmcpy() const { return m_secconfrmcpy ; }
   void SetSecconfrmcpy(bool value) { m_secconfrmcpy = value ; }
 
+  bool GetSeclockonmin() const { return m_seclockonmin ; }
+  void SetSeclockonmin(bool value) { m_seclockonmin = value ; }
+
   bool GetSeclockonwinlock() const { return m_seclockonwinlock ; }
   void SetSeclockonwinlock(bool value) { m_seclockonwinlock = value ; }
 
-  bool GetSysstartup() const { return m_sysstartup ; }
-  void SetSysstartup(bool value) { m_sysstartup = value ; }
+  int GetShiftdoubleclickaction() const { return m_shiftdoubleclickaction ; }
+  void SetShiftdoubleclickaction(int value) { m_shiftdoubleclickaction = value ; }
+
+  bool GetShownotesastipsinviews() const { return m_shownotesastipsinviews ; }
+  void SetShownotesastipsinviews(bool value) { m_shownotesastipsinviews = value ; }
+
+  bool GetShowusernameintree() const { return m_showusernameintree ; }
+  void SetShowusernameintree(bool value) { m_showusernameintree = value ; }
+
+  bool GetSysdefopenro() const { return m_sysdefopenro ; }
+  void SetSysdefopenro(bool value) { m_sysdefopenro = value ; }
 
   int GetSysmaxmru() const { return m_sysmaxmru ; }
   void SetSysmaxmru(int value) { m_sysmaxmru = value ; }
@@ -304,14 +289,20 @@ public:
   bool GetSysmruonfilemenu() const { return m_sysmruonfilemenu ; }
   void SetSysmruonfilemenu(bool value) { m_sysmruonfilemenu = value ; }
 
-  bool GetSysdefopenro() const { return m_sysdefopenro ; }
-  void SetSysdefopenro(bool value) { m_sysdefopenro = value ; }
-
   bool GetSysmultinst() const { return m_sysmultinst ; }
   void SetSysmultinst(bool value) { m_sysmultinst = value ; }
 
-  bool GetPutgroups1st() const { return m_putgroups1st ; }
-  void SetPutgroups1st(bool value) { m_putgroups1st = value ; }
+  bool GetSysstartup() const { return m_sysstartup ; }
+  void SetSysstartup(bool value) { m_sysstartup = value ; }
+
+  bool GetUsedefuser() const { return m_usedefuser ; }
+  void SetUsedefuser(bool value) { m_usedefuser = value ; }
+
+  bool GetWordwrapnotes() const { return m_wordwrapnotes ; }
+  void SetWordwrapnotes(bool value) { m_wordwrapnotes = value ; }
+
+  bool GetUseAltAutoType() const { return m_useAltAutoType ; }
+  void SetUseAltAutoType(bool value) { m_useAltAutoType = value ; }
 
   /// Retrieves bitmap resources
   wxBitmap GetBitmapResource( const wxString& name );
@@ -338,24 +329,9 @@ public:
   wxCheckBox* m_preexpirywarnCB;
   wxSpinCtrl* m_preexpirywarndaysSB;
   wxComboBox* m_DCACB;
+  wxComboBox* m_SDCACB;
   wxTextCtrl* m_defusernameTXT;
   wxStaticText* m_defusernameLBL;
-  wxGridSizer* m_pwMinsGSzr;
-  wxCheckBox* m_pwpUseLowerCtrl;
-  wxBoxSizer* m_pwNumLCbox;
-  wxSpinCtrl* m_pwpLCSpin;
-  wxCheckBox* m_pwpUseUpperCtrl;
-  wxBoxSizer* m_pwNumUCbox;
-  wxSpinCtrl* m_pwpUCSpin;
-  wxCheckBox* m_pwpUseDigitsCtrl;
-  wxBoxSizer* m_pwNumDigbox;
-  wxSpinCtrl* m_pwpDigSpin;
-  wxCheckBox* m_pwpSymCtrl;
-  wxBoxSizer* m_pwNumSymbox;
-  wxSpinCtrl* m_pwpSymSpin;
-  wxCheckBox* m_pwpEasyCtrl;
-  wxCheckBox* m_pwpPronounceCtrl;
-  wxCheckBox* m_pwpHexCtrl;
   wxCheckBox* m_pwhistsaveCB;
   wxSpinCtrl* m_pwhistnumdfltSB;
   wxButton* m_pwhistapplyBN;
@@ -365,41 +341,44 @@ public:
   wxSpinCtrl* m_sysmaxREitemsSB;
   wxString m_otherbrowserparams;
 private:
-  bool m_saveimmediate;
-  bool m_backupb4save;
   bool m_alwaysontop;
-  bool m_showusernameintree;
-  bool m_shownotesastipsinviews;
-  bool m_pwshowinedit;
-  bool m_wordwrapnotes;
-  int m_inittreeview;
-  bool m_preexpirywarn;
-  bool m_notesshowinedit;
-  bool m_confirmdelete;
-  bool m_maintaindatetimestamps;
-  bool m_escexits;
-  int m_doubleclickaction;
-  bool m_minauto;
   wxString m_autotypeStr;
-  bool m_usedefuser;
-  bool m_querysetdef;
+  bool m_backupb4save;
+  bool m_confirmdelete;
+  int m_doubleclickaction;
+  bool m_escexits;
+  int m_hashIterSlider;
+  int m_inittreeview;
+  bool m_maintaindatetimestamps;
+  bool m_minauto;
+  bool m_notesshowinedit;
   wxString m_otherbrowser;
+  bool m_preexpirywarn;
+  bool m_putgroups1st;
   int m_pwdefaultlength;
-  bool m_secclrclponmin;
+  bool m_pwshowinedit;
+  bool m_querysetdef;
+  bool m_saveimmediate;
   bool m_secclrclponexit;
-  bool m_seclockonmin;
+  bool m_secclrclponmin;
   bool m_secconfrmcpy;
+  bool m_seclockonmin;
   bool m_seclockonwinlock;
-  bool m_sysstartup;
+  int m_shiftdoubleclickaction;
+  bool m_shownotesastipsinviews;
+  bool m_showusernameintree;
+  bool m_sysdefopenro;
   int m_sysmaxmru;
   bool m_sysmruonfilemenu;
-  bool m_sysdefopenro;
   bool m_sysmultinst;
-  bool m_putgroups1st;
+  bool m_sysstartup;
+  bool m_usedefuser;
+  bool m_wordwrapnotes;
+  bool m_useAltAutoType;
+////@end COptions member variables
 #if defined(__X__) || defined(__WXGTK__)
   bool m_usePrimarySelection;
 #endif
-////@end COptions member variables
  private:
   void PrefsToPropSheet();
   void PropSheetToPrefs();
