@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+ * Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -27,10 +27,6 @@
 
 #include <iostream> // currently for debugging
 
-#if !defined(_WIN32)
-#include <unistd.h> // for fork()
-#endif
-
 using namespace std;
 
 #include "pwsafeapp.h"
@@ -54,6 +50,8 @@ using namespace std;
 #include "../../core/PWSLog.h"
 #include "./pwsmenushortcuts.h"
 
+#include <wx/spinctrl.h>
+
 #ifdef __WXMSW__
 #include <wx/msw/msvcrt.h>
 #endif
@@ -71,6 +69,22 @@ using namespace std;
 #define STR(s) wxT(s)
 #endif
 
+
+// wx debug messages (a) don't really interest us, and
+// (b) manage to crash wx 3.0.2 under Windows due to some
+// odd initialization sequence error, so we just shut them up.
+// Need to do this in a global as well, as we die in another
+// global trace in Release Windows build
+
+class RestrainLog {
+public:
+  RestrainLog(wxLogLevel level) {
+    wxLog::SetComponentLevel(wxT("wx"), level);
+  }
+};
+
+static RestrainLog restLog(wxLOG_Info);
+
 static const wxCmdLineEntryDesc cmdLineDesc[] = {
   {wxCMD_LINE_SWITCH, STR("?"), STR("help"),
    STR("displays command line usage"),
@@ -82,10 +96,10 @@ static const wxCmdLineEntryDesc cmdLineDesc[] = {
    STR("validate (and repair) database"),
    wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL},
   {wxCMD_LINE_OPTION, STR("e"), STR("encrypt"),
-   STR("encrypt a file (deprecated)"),
+   STR("encrypt a file"),
    wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL},
   {wxCMD_LINE_OPTION, STR("d"), STR("decrypt"),
-   STR("decrypt a file (deprecated)"),
+   STR("decrypt a file"),
    wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL},
   {wxCMD_LINE_SWITCH, STR("c"), STR("close"),
    STR("start 'closed', without prompting for database"),
@@ -136,82 +150,23 @@ IMPLEMENT_CLASS( PwsafeApp, wxApp )
  * PwsafeApp event table definition
  */
 
-BEGIN_EVENT_TABLE( PwsafeApp, wxApp )
+  BEGIN_EVENT_TABLE( PwsafeApp, wxApp )
 
 ////@begin PwsafeApp event table entries
 ////@end PwsafeApp event table entries
-EVT_ACTIVATE_APP(PwsafeApp::OnActivate)
-EVT_TIMER(ACTIVITY_TIMER_ID, PwsafeApp::OnActivityTimer)
-EVT_CUSTOM(wxEVT_GUI_DB_PREFS_CHANGE, wxID_ANY, PwsafeApp::OnDBGUIPrefsChange)
-END_EVENT_TABLE()
-
-
-
-void PwsafeApp::initLanguageSupport()
-{
-  long language = wxLocale::GetSystemLanguage();
-#if defined(__UNIX__) && !defined(__WXMAC__)
-  // Workaround for wx bug 15006
-  if (language == wxLANGUAGE_UNKNOWN) {
-    std::wcerr << L"Couldn't detect locale. Trying to skip empty env. variables."
-		 << endl;
-    // Undefine empty environment variables and try again
-    wxString langFull;
-    if (wxGetEnv(wxT("LC_ALL"), &langFull) && !langFull){
-      wxUnsetEnv(wxT("LC_ALL"));
-      std::wcerr << L"Empty LC_ALL variable was dropped" << endl;
-    }
-    if (wxGetEnv(wxT("LC_MESSAGES"), &langFull) && !langFull){
-      wxUnsetEnv(wxT("LC_MESSAGES"));
-      std::wcerr << L"Empty LC_MESSAGES variable was dropped" << endl;
-    }
-    if (wxGetEnv(wxT("LANG"), &langFull) && !langFull){
-      wxUnsetEnv(wxT("LANG"));
-      std::wcerr << L"Empty LANG variable was dropped" << endl;
-    }
-    language = wxLocale::GetSystemLanguage();
-  }
-#endif
-  // load language if possible, fall back to english otherwise
-  if(wxLocale::IsAvailable(language)) {
-    m_locale = new wxLocale( language, wxLOCALE_CONV_ENCODING );
-
-    // add locale search paths
-    m_locale->AddCatalogLookupPathPrefix(wxT("/usr"));
-    m_locale->AddCatalogLookupPathPrefix(wxT("/usr/local"));
-#if defined(__WXDEBUG__) || defined(_DEBUG) || defined(DEBUG)
-    m_locale->AddCatalogLookupPathPrefix(wxT("../I18N/mos"));
-#endif
-    if (!m_locale->AddCatalog(wxT("pwsafe"))) {
-      std::wcerr << L"Couldn't load text for "
-		 << m_locale->GetLanguageName(language).c_str() << endl;
-    }
-
-    if(! m_locale->IsOk()) {
-      std::cerr << "selected language is wrong" << std::endl;
-      delete m_locale;
-      m_locale = new wxLocale( wxLANGUAGE_ENGLISH );
-      language = wxLANGUAGE_ENGLISH;
-    }
-  } else {
-    std::cerr << "The selected language is not supported by your system."
-	      << "Try installing support for this language." << std::endl;
-    m_locale = new wxLocale( wxLANGUAGE_ENGLISH );
-    language = wxLANGUAGE_ENGLISH;
-  }
-  // add translation for standard resources
-  m_locale->AddCatalog(wxT("wxstd"));
-}
+  EVT_TIMER(IDLE_TIMER_ID, PwsafeApp::OnIdleTimer)
+  EVT_CUSTOM(wxEVT_GUI_DB_PREFS_CHANGE, wxID_ANY, PwsafeApp::OnDBGUIPrefsChange)
+  END_EVENT_TABLE()
 
 /*!
  * Constructor for PwsafeApp
  */
 
-PwsafeApp::PwsafeApp() : m_activityTimer(new wxTimer(this, ACTIVITY_TIMER_ID)),
-			 m_frame(0), m_recentDatabases(0),
-			 m_controller(new wxHtmlHelpController), m_locale(NULL)
+PwsafeApp::PwsafeApp() : m_idleTimer(new wxTimer(this, IDLE_TIMER_ID)),
+                         m_idleFlag(true), m_frame(0), m_recentDatabases(0),
+       m_controller(new wxHtmlHelpController), m_locale(NULL)
 {
-    Init();
+  Init();
 }
 
 /*!
@@ -219,7 +174,7 @@ PwsafeApp::PwsafeApp() : m_activityTimer(new wxTimer(this, ACTIVITY_TIMER_ID)),
  */
 PwsafeApp::~PwsafeApp()
 {
-  delete m_activityTimer;
+  delete m_idleTimer;
   delete m_recentDatabases;
 
   PWSprefs::DeleteInstance();
@@ -236,17 +191,40 @@ PwsafeApp::~PwsafeApp()
 
 void PwsafeApp::Init()
 {
-  initLanguageSupport();
+  m_locale = new wxLocale;
+  wxLocale::AddCatalogLookupPathPrefix( wxT("/usr/share/locale") );
+  wxLocale::AddCatalogLookupPathPrefix( wxT("/usr") );
+  wxLocale::AddCatalogLookupPathPrefix( wxT("/usr/local") );
+#if defined(__WXDEBUG__) || defined(_DEBUG) || defined(DEBUG)
+  wxLocale::AddCatalogLookupPathPrefix( wxT("../I18N/mos") );
+#endif
+
 ////@begin PwsafeApp member initialisation
 ////@end PwsafeApp member initialisation
 }
 
+#ifdef __WXDEBUG__
+void PwsafeApp::OnAssertFailure(const wxChar *file, int line, const wxChar *func,
+                const wxChar *cond, const wxChar *msg)
+{
+  if (m_locale)
+    wxApp::OnAssertFailure(file, line, func, cond, msg);
+  else
+      std::wcerr << file << L'(' << line << L"):"
+                 << L" assert \"" << cond << L"\" failed in " << (func? func: L"") << L"(): "
+                 << (msg? msg: L"") << endl;
+}
+#endif
+
 /*!
  * Initialisation for PwsafeApp
  */
 
 bool PwsafeApp::OnInit()
 {
+  m_locale->Init( GetSystemLanguage() );
+  ActivateLanguage( GetSystemLanguage() );
+
   SetAppName(pwsafeAppName);
   m_core.SetApplicationNameAndVersion(tostdstring(pwsafeAppName),
                                       DWORD((MINORVERSION << 16) | MAJORVERSION));
@@ -323,19 +301,6 @@ bool PwsafeApp::OnInit()
   m_core.SetApplicationNameAndVersion(tostdstring(progName),
                                       MAKEWORD(MINORVERSION, MAJORVERSION));
 
-#if !defined(__WXDEBUG__) && !defined(__WXMAC__) && !defined(_WIN32)
-  // Now's a good time to fork
-  // and exit the parent process, returning the command prompt to the user
-  // (but not for debug builds - just make debugging harder)
-  pid_t pid = fork();
-  if (pid == -1) {
-    perror("fork"); // should never happen!
-    exit(1);
-  } else if (pid != 0) { // parent
-    exit(0);
-  }
-#endif /* _DEBUG */
-
   static wxSingleInstanceChecker appInstance;
   if (!prefs->GetPref(PWSprefs::MultipleInstances) &&
         (appInstance.Create(wxT("pwsafe.lck"), towxstring(pws_os::getuserprefsdir())) &&
@@ -355,13 +320,13 @@ bool PwsafeApp::OnInit()
 
   if (cmd_closed) {
     m_core.SetCurFile(wxT(""));
-    // dbox.SetStartClosed(true);
   }
-  // dbox.SetStartSilent(cmd_silent);
   if (cmd_minimized) {
     m_core.SetCurFile(wxT(""));
-    //dbox.SetStartClosed(true);
-    // dbox.SetStartSilent(true);
+  }
+  if (cmd_silent) {
+    // start silent implies use system tray.
+    PWSprefs::GetInstance()->SetPref(PWSprefs::UseSystemTray, true);
   }
 
   //Initialize help subsystem
@@ -377,7 +342,7 @@ bool PwsafeApp::OnInit()
   m_appIcons.AddIcon(pwsafe32);
   m_appIcons.AddIcon(pwsafe48);
 
-  if (!cmd_closed) {
+  if (!cmd_closed && !cmd_silent && !cmd_minimized) {
     // Get the file, r/w mode and password from user
     // Note that file may be new
     CSafeCombinationEntry* initWindow = new CSafeCombinationEntry(NULL, m_core);
@@ -398,25 +363,76 @@ bool PwsafeApp::OnInit()
 
   RestoreFrameCoords();
   m_frame->Show();
-  SetTopWindow(m_frame);
+  if (cmd_minimized)
+    m_frame->Iconize(true);
+  else if (cmd_silent) {
+    m_frame->SetTrayStatus(true);
+    m_frame->HideUI(true);
+  } else
+    SetTopWindow(m_frame);
   return true;
 }
 
-void PwsafeApp::StopIdleTimer()
+/*!
+ * Initializing the language support means currently
+ * to determine the system default language and
+ * activate this one for the application.
+ */
+wxLanguage PwsafeApp::GetSystemLanguage()
 {
-  m_activityTimer->Stop();
+  int language = wxLocale::GetSystemLanguage();
+
+#if defined(__UNIX__) && !defined(__WXMAC__) && wxCHECK_VERSION( 2, 8, 0 )
+  // Workaround for wx bug 15006 that has been fixed in wxWidgets 2.8
+  if (language == wxLANGUAGE_UNKNOWN) {
+    std::wcerr << L"Couldn't detect locale. Trying to skip empty env. variables." << std::endl;
+    // Undefine empty environment variables and try again
+    wxString langFull;
+    if (wxGetEnv(wxT("LC_ALL"), &langFull) && !langFull) {
+      wxUnsetEnv(wxT("LC_ALL"));
+      std::wcerr << L"Empty LC_ALL variable was dropped" << std::endl;
+    }
+    if (wxGetEnv(wxT("LC_MESSAGES"), &langFull) && !langFull) {
+      wxUnsetEnv(wxT("LC_MESSAGES"));
+      std::wcerr << L"Empty LC_MESSAGES variable was dropped" << std::endl;
+    }
+    if (wxGetEnv(wxT("LANG"), &langFull) && !langFull) {
+      wxUnsetEnv(wxT("LANG"));
+      std::wcerr << L"Empty LANG variable was dropped" << std::endl;
+    }
+    language = wxLocale::GetSystemLanguage();
+  }
+#endif
+
+  return static_cast<wxLanguage>(language);
 }
 
-void PwsafeApp::StartIdleTimer()
+/*!
+ * Activates a language.
+ *
+ * \param language the wxWidgets specific enumeration of type wxLanguage representing a supported language
+ * \see http://docs.wxwidgets.org/trunk/language_8h.html#a7d1c74ce43b2fb7acf7a6fa438c0ee86
+ */
+bool PwsafeApp::ActivateLanguage(wxLanguage language)
 {
-  int timeout = PWSprefs::GetInstance()->GetPref(PWSprefs::IdleTimeout);
-  if (timeout != 0)
-    m_activityTimer->Start(timeout*60*1000, true);
-}
+  static wxString DOMAIN_("pwsafe");
 
-bool PwsafeApp::IsIdleTimerRunning() const
-{
-  return m_activityTimer->IsRunning();
+  wxTranslations *translations = new wxTranslations;
+  translations->SetLanguage( language );
+  translations->AddStdCatalog();
+  wxTranslations::Set( translations ); // takes care of occupied memory by wxTranslations
+
+  if ( language == wxLANGUAGE_ENGLISH) {
+    translations->SetLanguage( wxLANGUAGE_ENGLISH );
+    return true;
+  }
+
+  if ( !translations->AddCatalog( DOMAIN_ ) ) {
+    std::wcerr << L"Couldn't load language catalog for " << wxLocale::GetLanguageName(language) << std::endl;
+    translations->SetLanguage( wxLANGUAGE_ENGLISH );
+  }
+
+  return translations->IsLoaded( DOMAIN_ );
 }
 
 /*!
@@ -425,7 +441,7 @@ bool PwsafeApp::IsIdleTimerRunning() const
 
 int PwsafeApp::OnExit()
 {
-  StopIdleTimer();
+  m_idleTimer->Stop();
   recentDatabases().Save();
   PWSprefs *prefs = PWSprefs::GetInstance();
   if (!m_core.GetCurFile().empty())
@@ -441,31 +457,45 @@ int PwsafeApp::OnExit()
 ////@end PwsafeApp cleanup
 }
 
-void PwsafeApp::OnActivate(wxActivateEvent& actEvent)
+void PwsafeApp::ConfigureIdleTimer()
 {
-  StopIdleTimer();
-  if (!actEvent.GetActive()) {
-    StartIdleTimer();
+  const PWSprefs *prefs =   PWSprefs::GetInstance();
+
+  bool isRunning = m_idleTimer->IsRunning();
+  bool shouldBeRunning = prefs->GetPref(PWSprefs::LockDBOnIdleTimeout);
+  int timeOut = shouldBeRunning ? prefs->GetPref(PWSprefs::IdleTimeout) : 0;
+
+  if (!isRunning) {
+    if (shouldBeRunning && timeOut != 0) {
+      m_idleTimer->Start(timeOut * 60 * 1000, wxTIMER_CONTINUOUS);
+    } else {
+      // not running, nor should it be - nop
+    }
+  } else { // running
+    if (!shouldBeRunning || timeOut == 0) {
+      m_idleTimer->Stop();
+    } else if (timeOut != 0) { // Restart
+      m_idleTimer->Start(timeOut * 60 * 1000, wxTIMER_CONTINUOUS);
+    }
   }
-  actEvent.Skip();
 }
 
-void PwsafeApp::OnActivityTimer(wxTimerEvent &evt)
+void PwsafeApp::OnIdleTimer(wxTimerEvent &evt)
 {
-  if (evt.GetId() == ACTIVITY_TIMER_ID && PWSprefs::GetInstance()->GetPref(PWSprefs::LockDBOnIdleTimeout)) {
-    if (m_frame != NULL && !m_frame->GetCurrentSafe().IsEmpty())
-      m_frame->HideUI(true);  //true => lock
+  if (evt.GetId() == IDLE_TIMER_ID && PWSprefs::GetInstance()->GetPref(PWSprefs::LockDBOnIdleTimeout)) {
+    if (m_frame != NULL && !m_frame->GetCurrentSafe().IsEmpty()) {
+      if (m_idleFlag) // cleared if a user event occurred via FilterEvent()
+        m_frame->HideUI(true);  //true => lock
+      else
+        m_idleFlag = true; // arm for next interval
+    }
   }
 }
 
 void PwsafeApp::OnDBGUIPrefsChange(wxEvent& evt)
 {
   UNREFERENCED_PARAMETER(evt);
-  if (IsIdleTimerRunning()) {
-    // Restart, in case Idle timer settings have just changed
-    StopIdleTimer();
-    StartIdleTimer();
-  }
+  ConfigureIdleTimer();
 }
 
 CRecentDBList &PwsafeApp::recentDatabases()
@@ -511,9 +541,85 @@ void PwsafeApp::RestoreFrameCoords(void)
 }
 
 int PwsafeApp::FilterEvent(wxEvent& evt) {
+  const wxEventType et = evt.GetEventType();
+
+  // Clear idle flag for lock-on-idle timer
+  // We need a whitelist rather than a blacklist because
+  // undocumented events are passed through here as well...
+  if ((et == wxEVT_COMMAND_BUTTON_CLICKED) ||
+      (et == wxEVT_COMMAND_CHECKBOX_CLICKED) ||
+      (et == wxEVT_COMMAND_CHOICE_SELECTED) ||
+      (et == wxEVT_COMMAND_LISTBOX_SELECTED) ||
+      (et == wxEVT_COMMAND_LISTBOX_DOUBLECLICKED) ||
+      (et == wxEVT_COMMAND_TEXT_UPDATED) ||
+      (et == wxEVT_COMMAND_TEXT_ENTER) ||
+      (et == wxEVT_COMMAND_MENU_SELECTED) ||
+      (et == wxEVT_COMMAND_SLIDER_UPDATED) ||
+      (et == wxEVT_COMMAND_RADIOBOX_SELECTED) ||
+      (et == wxEVT_COMMAND_RADIOBUTTON_SELECTED) ||
+      (et == wxEVT_COMMAND_SCROLLBAR_UPDATED) ||
+      (et == wxEVT_COMMAND_VLBOX_SELECTED) ||
+      (et == wxEVT_COMMAND_COMBOBOX_SELECTED) ||
+      (et == wxEVT_COMMAND_TOOL_RCLICKED) ||
+      (et == wxEVT_COMMAND_TOOL_ENTER) ||
+      (et == wxEVT_COMMAND_SPINCTRL_UPDATED) ||
+      (et == wxEVT_LEFT_DOWN) ||
+      (et == wxEVT_LEFT_UP) ||
+      (et == wxEVT_MIDDLE_DOWN) ||
+      (et == wxEVT_MIDDLE_UP) ||
+      (et == wxEVT_RIGHT_DOWN) ||
+      (et == wxEVT_RIGHT_UP) ||
+      (et == wxEVT_MOTION) ||
+      (et == wxEVT_ENTER_WINDOW) ||
+      (et == wxEVT_LEAVE_WINDOW) ||
+      (et == wxEVT_LEFT_DCLICK) ||
+      (et == wxEVT_MIDDLE_DCLICK) ||
+      (et == wxEVT_RIGHT_DCLICK) ||
+      (et == wxEVT_SET_FOCUS) ||
+      (et == wxEVT_MOUSEWHEEL) ||
+      (et == wxEVT_NAVIGATION_KEY) ||
+      (et == wxEVT_KEY_DOWN) ||
+      (et == wxEVT_KEY_UP) ||
+      (et == wxEVT_SCROLL_TOP) ||
+      (et == wxEVT_SCROLL_BOTTOM) ||
+      (et == wxEVT_SCROLL_LINEUP) ||
+      (et == wxEVT_SCROLL_LINEDOWN) ||
+      (et == wxEVT_SCROLL_PAGEUP) ||
+      (et == wxEVT_SCROLL_PAGEDOWN) ||
+      (et == wxEVT_SCROLL_THUMBTRACK) ||
+      (et == wxEVT_SCROLL_THUMBRELEASE) ||
+      (et == wxEVT_SCROLL_CHANGED) ||
+      (et == wxEVT_SIZE) ||
+      (et == wxEVT_MOVE) ||
+      (et == wxEVT_ACTIVATE_APP) ||
+      (et == wxEVT_ACTIVATE) ||
+      (et == wxEVT_SHOW) ||
+      (et == wxEVT_ICONIZE) ||
+      (et == wxEVT_MAXIMIZE) ||
+      (et == wxEVT_MENU_OPEN) ||
+      (et == wxEVT_MENU_CLOSE) ||
+      (et == wxEVT_MENU_HIGHLIGHT) ||
+      (et == wxEVT_CONTEXT_MENU) ||
+      (et == wxEVT_JOY_BUTTON_DOWN) ||
+      (et == wxEVT_JOY_BUTTON_UP) ||
+      (et == wxEVT_JOY_MOVE) ||
+      (et == wxEVT_JOY_ZMOVE) ||
+      (et == wxEVT_DROP_FILES) ||
+      (et == wxEVT_COMMAND_TEXT_COPY) ||
+      (et == wxEVT_COMMAND_TEXT_CUT) ||
+      (et == wxEVT_COMMAND_TEXT_PASTE) ||
+      (et == wxEVT_COMMAND_LEFT_CLICK) ||
+      (et == wxEVT_COMMAND_LEFT_DCLICK) ||
+      (et == wxEVT_COMMAND_RIGHT_CLICK) ||
+      (et == wxEVT_COMMAND_RIGHT_DCLICK) ||
+      (et == wxEVT_COMMAND_ENTER) ||
+      (et == wxEVT_HELP) ||
+      (et == wxEVT_DETAILED_HELP))
+    m_idleFlag = false; // for lock on idle timer
+
   if (evt.IsCommandEvent() && evt.GetId() == wxID_HELP &&
-          (evt.GetEventType() == wxEVT_COMMAND_BUTTON_CLICKED ||
-            evt.GetEventType() == wxEVT_COMMAND_MENU_SELECTED)) {
+      (et == wxEVT_COMMAND_BUTTON_CLICKED ||
+       et == wxEVT_COMMAND_MENU_SELECTED)) {
     OnHelp(*wxDynamicCast(&evt, wxCommandEvent));
     return int(true);
   }
